from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.model import Friend
from graia.ariadne.message.chain import MessageChain
from graia.ariadne.event.message import FriendMessage
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.message.parser.twilight import (
    Twilight,
    FullMatch,
    RegexResult,
    WildcardMatch,
)

from ....core.control import Permission
from ....core.group_config import GroupPermission

channel = Channel.current()


@channel.use(
    ListenerSchema(
        listening_events=[FriendMessage],
        inline_dispatchers=[
            Twilight(
                [FullMatch("添加vip"), "groupid" @ WildcardMatch(optional=True)],
            )
        ],
    )
)
async def main(app: Ariadne, friend: Friend, groupid: RegexResult):
    Permission.manual(friend, Permission.MASTER)
    if groupid.matched:
        say = groupid.result.display
        if say.isdigit():
            gp = GroupPermission(int(say))
            if gp.add_to_vips():
                await app.send_friend_message(friend, MessageChain("成功将群设定为 vip 群"))
            else:
                await app.send_friend_message(friend, MessageChain("该群已是 vip 群"))
        else:
            await app.send_friend_message(friend, MessageChain("群号仅可为数字"))
    else:
        await app.send_friend_message(friend, MessageChain("未输入群号"))



from loguru import logger
from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.model import Group
from graia.ariadne.message.element import At
from graia.ariadne.event.message import GroupMessage
from graia.ariadne.message.chain import MessageChain
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.message.parser.twilight import (
    Twilight,
    RegexMatch,
    ElementMatch,
    ElementResult,
)

from ....core.bot_config import BotConfig
from ....utils.text2image import text2image
from ....core.control import Interval, Permission

channel = Channel.current()
menu_image = None
help_text = (
    "BBot 功能菜单：\n"
    "=================================================================\n"
    " 0. /quit\n"
    " 1. BiliBili 视频、专栏内容解析，支持 OpenAI 总结\n"
    " 2. 查看关注列表\n"
    f" 3. @{BotConfig.name} 关注 <uid>\n    > @{BotConfig.name} (订阅|关注)(主播|[uU][pP])?\n"
    f" 4. @{BotConfig.name} 取关 <uid>\n    > @{BotConfig.name} (退订|取消?关注?)\\s?(主播|[uU][pP])?\n"
    " 5. 查看动态 <uid> <offset>\n    > offset 可以填写数字，用来代表该 UP 的倒数第几条动态\n"
    " 6. 设定|删除 昵称 <uid> [昵称]\n"
    " 7. 开启|关闭 @全体成员 <uid>\n"
    "=================================================================\n"
    "BBot 采用 gRPC 接口进行动态检查。"
)

if BotConfig.Bilibili.use_login:
    help_text += "关注后目标 UP 将粉丝数 +1，收到动态更新后 BBot 将会对动态点赞。"


@channel.use(
    ListenerSchema(
        listening_events=[GroupMessage],
        inline_dispatchers=[
            Twilight(
                [
                    "at" @ ElementMatch(At, optional=True),
                    RegexMatch(r"([/.。?？!！])?(帮助|菜单|功能|help|menu)([/.。?？!！])?"),
                ]
            )
        ],
        decorators=[Permission.require(), Interval.require()],
        priority=15,
    )
)
async def main(app: Ariadne, group: Group, at: ElementResult):
    if at.result:
        at_element: At = at.result  # type: ignore
        if at_element.target != BotConfig.Mirai.account:
            return

    global menu_image

    if not menu_image:
        logger.info("正在生成菜单图片")
        menu_image = await app.upload_image(await text2image(help_text))

    await app.send_group_message(group, MessageChain(menu_image))



import asyncio
import secrets

from loguru import logger
from typing import Optional
from datetime import timezone
from jose import JWTError, jwt
from graia.ariadne.app import Ariadne
from datetime import datetime, timedelta
from graia.ariadne.model import MemberPerm, Group
from graia.ariadne.exception import UnknownTarget
from fastapi.security import OAuth2PasswordBearer
from graia.amnesia.builtins.memcache import Memcache
from fastapi import APIRouter, Depends, HTTPException, status

from ....core.control import Permission
from ....core.data import get_sub_by_group
from ....core.group_config import GroupPermission
from ....model.fastapi import AuthResponse, Token, KeyResponse, InfoResponse, Info, GroupItem

SECRET_KEY = secrets.token_hex(64)
ALGORITHM = "HS256"
ACCESS_TOKEN_EXPIRE_DAY = 7

router = APIRouter(tags=["Auth"])
oauth2_scheme = OAuth2PasswordBearer(tokenUrl="/api/auth/verify_key")
logger.info(f"已生成 API 密钥: {SECRET_KEY[:8]}{'*' * 20}{SECRET_KEY[-8:]}")


class UnauthorizedException(HTTPException):
    def __init__(self, detail: str = "Unauthorized"):
        super().__init__(
            status_code=status.HTTP_401_UNAUTHORIZED,
            detail=detail,
            headers={"WWW-Authenticate": "Bearer"},
        )


def create_access_token(self):
    expire = datetime.now(timezone.utc) + timedelta(days=ACCESS_TOKEN_EXPIRE_DAY)
    to_encode = {"exp": expire, "sub": self}
    return jwt.encode(
        to_encode, SECRET_KEY, algorithm=ALGORITHM, headers={"typ": "JWT", "alg": ALGORITHM}
    )


def decode_token(token: str):
    payload = jwt.decode(token, SECRET_KEY, algorithms=ALGORITHM)
    qq = payload.get("sub", "")
    return Info(
        qq=qq,
        permission=Permission.get(qq),
        token=Token(access_token=token, expires_at=payload.get("exp")),
    )


async def verify_token(token: str = Depends(oauth2_scheme)):
    try:
        return decode_token(token)
    except JWTError as e:
        raise UnauthorizedException(detail="Invalid token") from e


async def get_user_groups(qq: Optional[int] = None, master: bool = False) -> list[GroupItem]:
    app = Ariadne.current()

    async def get_group(group: Group):
        try:
            if (await app.get_member(group.id, qq or 0)).permission in [
                MemberPerm.Administrator,
                MemberPerm.Owner,
            ]:
                return group
        except UnknownTarget:
            return

    async def get_group_item(group: Group):
        return GroupItem(
            id=group.id,
            name=group.name,
            member_count=len(await app.get_member_list(group)),
            follow_count=len(get_sub_by_group(group.id)),
            is_vip=GroupPermission(group).is_vip(),
        )

    group_list = await app.get_group_list()

    return await asyncio.gather(
        *[
            get_group_item(x)
            for x in (
                group_list
                if master
                else [
                    i
                    for i in await asyncio.gather(*[get_group(group) for group in group_list])
                    if i
                ]
            )
        ]
    )


async def can_manage_groups(info: Info):
    if info.permission < Permission.MASTER:
        return await get_user_groups(int(info.qq))
    return await get_user_groups(master=True)


async def can_manage_group(info: Info, group: int):
    if info.permission > Permission.MASTER:
        return True
    app = Ariadne.current()
    try:
        can = (await app.get_member(group, int(info.qq) or 0)).permission in [
            MemberPerm.Administrator,
            MemberPerm.Owner,
        ]
    except UnknownTarget:
        can = False
    return can


@router.get("/get_key", response_model=KeyResponse, summary="获取 key")
async def get_key():
    app = Ariadne.current()
    memcache = app.launch_manager.get_interface(Memcache)
    key = secrets.token_urlsafe(16)
    await memcache.set(key, False, timedelta(seconds=120))
    return KeyResponse(data=key)


@router.get("/verify_key", response_model=AuthResponse, summary="验证 key")
async def verify_key(key: str):
    app = Ariadne.current()
    memcache = app.launch_manager.get_interface(Memcache)
    if await memcache.has(key):
        if qq := await memcache.get(key):
            await memcache.delete(key)
            return AuthResponse(
                code=status.HTTP_200_OK,
                message="",
                data=await verify_token(create_access_token(qq)),
            )
        else:
            return AuthResponse(
                code=status.HTTP_102_PROCESSING, message="Waiting for verification", data=None
            )
    else:
        return AuthResponse(code=status.HTTP_404_NOT_FOUND, message="Key not found", data=None)


@router.get("/whoami", response_model=InfoResponse, summary="获取 token 信息")
async def whoami(info: Info = Depends(verify_token)):
    return InfoResponse(data=info)



import contextlib

from loguru import logger
from sentry_sdk import init as sentry_sdk_init

sentry_sdk_init(
    dsn="https://e7455ef7813c42e2b854bdd5c26adeb6@o1418272.ingest.sentry.io/6761179",
    traces_sample_rate=1.0,
    release="1.4.1",
)


def run_bot():

    from ..bot import app

    with contextlib.suppress(Exception):
        app.launch_blocking()
    logger.info("BBot is shut down.")



import json
import httpx

from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.model import Group
from graia.ariadne.message.element import Image
from graia.ariadne.message.chain import MessageChain
from graia.ariadne.event.message import GroupMessage
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.message.parser.twilight import Twilight, FullMatch

from .core.control import Permission
from .utils.bilibili_request import grpc_get_playview

channel = Channel.current()


@channel.use(
    ListenerSchema(
        listening_events=[GroupMessage],
        inline_dispatchers=[Twilight([FullMatch("/test")])],
        decorators=[Permission.require(Permission.MASTER)],
    )
)
async def main(app: Ariadne, group: Group):
    print(await grpc_get_playview(439357034, 1076145927))



from typing import Union
from loguru import logger
from graia.saya import Channel
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.event.lifecycle import AccountLaunch, AccountShutdown, AccountConnectionFail

from ...core import BOT_Status, Status

channel = Channel.current()


@channel.use(
    ListenerSchema(listening_events=[AccountLaunch, AccountShutdown, AccountConnectionFail])
)
async def mirai_disconnect(event: Union[AccountLaunch, AccountShutdown, AccountConnectionFail]):
    if isinstance(event, AccountLaunch):
        if not BOT_Status.check_status(Status.ACCOUNT_CONNECTED):
            logger.info("[Mirai] 账号已连接成功")
            BOT_Status.set_status(Status.ACCOUNT_CONNECTED, True)
    elif BOT_Status.check_status(Status.ACCOUNT_CONNECTED):
        logger.warning("[Mirai] 账号断开")
        BOT_Status.set_status(Status.ACCOUNT_CONNECTED, False)



from typing import Optional
from pydantic import BaseModel


class TokenUsage(BaseModel):
    prompt_tokens: int
    completion_tokens: int
    total_tokens: int


class OpenAI(BaseModel):
    error: bool = False
    message: str = ""
    response: str = ""
    token_usage: Optional[TokenUsage] = None
    raw: dict = {}



import json

from pathlib import Path
from typing import Union
from graia.ariadne.model import Group

from .bot_config import BotConfig


grouplist_file = Path("data/group_list.json")


if grouplist_file.exists():
    data = json.loads(grouplist_file.read_text(encoding="utf-8"))
    whitelist = data["white"]
    vipgroup = data["vip"]
else:
    whitelist = []
    vipgroup = []
    grouplist_file.write_text(json.dumps({"white": [], "vip": []}, indent=2))


class GroupPermission:
    def __init__(self, group: Union[str, int, Group]):
        self.group_id = group.id if isinstance(group, Group) else int(group)

    def can_join(self):
        return self.group_id in whitelist if BotConfig.access_control else True

    def add_to_whitelist(self):
        if self.group_id in whitelist:
            return False
        whitelist.append(self.group_id)
        grouplist_file.write_text(json.dumps({"white": whitelist, "vip": vipgroup}, indent=2))
        return True

    def remove_from_whitelist(self):
        if self.group_id not in whitelist:
            return False
        whitelist.remove(self.group_id)
        grouplist_file.write_text(json.dumps({"white": whitelist, "vip": vipgroup}, indent=2))
        return True

    def is_vip(self):
        return self.group_id in vipgroup

    def add_to_vips(self):
        if self.group_id in vipgroup:
            return False
        vipgroup.append(self.group_id)
        grouplist_file.write_text(json.dumps({"white": whitelist, "vip": vipgroup}, indent=2))
        return True

    def remove_from_vips(self):
        if self.group_id not in vipgroup:
            return False
        vipgroup.remove(self.group_id)
        grouplist_file.write_text(json.dumps({"white": whitelist, "vip": vipgroup}, indent=2))
        return True



import sys
import json
import asyncio

from creart import it
from pathlib import Path
from loguru import logger
from graia.saya import Channel
from bilireq.login import Login
from grpc.aio import AioRpcError
from graia.ariadne.app import Ariadne
from graia.ariadne.model import Friend
from graia.broadcast.interrupt import Waiter
from graia.ariadne.message.element import Image
from bilireq.exceptions import ResponseCodeError
from graia.ariadne.message.chain import MessageChain
from graia.ariadne.event.message import FriendMessage
from graia.broadcast.interrupt import InterruptControl
from graia.ariadne.event.lifecycle import AccountLaunch
from graia.saya.builtins.broadcast.schema import ListenerSchema
from bilireq.grpc.dynamic import grpc_get_followed_dynamic_users, grpc_get_user_dynamics

from ...core.bot_config import BotConfig
from ...utils.up_operation import delete_uid
from ...core.data import get_all_uid, delete_sub_by_uid
from ...core import BOT_Status, Bili_Auth, Status, cache
from ...utils.bilibili_request import relation_modify, grpc_get_followed_dynamics_noads


inc = it(InterruptControl)
channel = Channel.current()
login_cache_file = Path("data/login_cache.json")


class CodeWaiter(Waiter.create([FriendMessage], block_propagation=True)):
    async def detected_event(self, app: Ariadne, friend: Friend, message: MessageChain):
        if friend.id == BotConfig.master:
            msg = message.display
            if msg.isdigit() and len(msg) == 6:
                return int(msg)
            else:
                await app.send_friend_message(BotConfig.master, "请输入正确的验证码")


async def init_dyn_id(up_uid):
    resp = await grpc_get_user_dynamics(int(up_uid))
    if resp:
        if len(resp.list) > 0:
            if len(resp.list) == 1:
                dyn = resp.list[0]
            elif resp.list[0].modules[0].module_author.is_top:
                dyn = resp.list[1]
            else:
                dyn = resp.list[0]

            up_name = dyn.modules[0].module_author.author.name
            dyn_id = int(dyn.extend.dyn_id_str)
            BOT_Status.offset[str(up_uid)] = dyn_id
            if dyn.modules[0].module_author.author.live.is_living == 1:
                logger.info(f"[BiliBili推送] {up_name}（{up_uid}） 已开播")
                BOT_Status.living[str(up_uid)] = 0
            logger.info(f"[BiliBili推送] UP {up_name}（{up_uid}） | {dyn_id}")
        else:
            delete_sub_by_uid(up_uid)
            logger.info(f"[BiliBili推送] UP {up_uid} 没有发布动态，已删除订阅")
    else:
        logger.warning(f"[BiliBili推送] UP {up_uid} 动态获取失败")


@channel.use(ListenerSchema(listening_events=[AccountLaunch]))
async def init(app: Ariadne):
    await asyncio.sleep(1)

    # 如果填写了用户名密码，则进行登录
    if BotConfig.Bilibili.username and BotConfig.Bilibili.password:
        logger.info("[BiliBili推送] 已填写 Bilibili 用户名密码，正在尝试登录")
        auth_data = None
        while True:
            await asyncio.sleep(1)
            if login_cache_file.exists():
                Bili_Auth.update(json.loads(login_cache_file.read_text()))
                try:
                    auth_data = await Bili_Auth.refresh()
                    logger.debug(auth_data)
                    logger.success("[Bilibili推送] 缓存登录完成")
                    await app.send_friend_message(
                        BotConfig.master,
                        MessageChain("[Bilibili推送] 缓存登录完成"),
                    )
                    break
                except ResponseCodeError as e:
                    logger.error(f"[Bilibili推送] 缓存登录失败：{e}，正在重新登录")
                    await app.send_friend_message(
                        BotConfig.master,
                        MessageChain(f"[Bilibili推送] 缓存登录失败：{e}，正在重新登录"),
                    )

            bilibili_login = Login()
            # 开始登录流程
            # 尝试用户名密码登录
            try:
                auth_data = await bilibili_login.pwd_login(
                    str(BotConfig.Bilibili.username), str(BotConfig.Bilibili.password)
                )
                logger.success("[Bilibili推送] 用户名密码登录完成")
                await app.send_friend_message(
                    BotConfig.master,
                    MessageChain("[Bilibili推送] 用户名密码登录完成"),
                )
            except ResponseCodeError as e:
                logger.warning(f"[BiliBili推送] 用户名密码登录失败：{e.code}，{e.msg}")
                logger.info("[BiliBili推送] 尝试使用短信验证码登录")
                try:
                    data = await bilibili_login.send_sms(BotConfig.Bilibili.username)
                    await app.send_friend_message(
                        BotConfig.master,
                        MessageChain(
                            f"[Bilibili推送] 用户名密码登录失败：{e.msg}\n正在使用验证码登录，请在 120 秒内发送验证码到此处"
                        ),
                    )
                    try:
                        code = await asyncio.wait_for(inc.wait(CodeWaiter()), timeout=120)
                        auth_data = await bilibili_login.sms_login(code)
                        logger.success("[BiliBili推送] 验证码登录完成")
                        await app.send_friend_message(
                            BotConfig.master, MessageChain("[BiliBili推送] 验证码登录完成")
                        )
                        break
                    except asyncio.TimeoutError:
                        logger.warning("[BiliBili推送] 等待输入验证码超时，如需重新登陆，请重启机器人")
                        await app.send_friend_message(
                            BotConfig.master,
                            MessageChain("[BiliBili推送] 等待输入验证码超时，如需重新登陆，请重启机器人"),
                        )
                        sys.exit(1)

                except ResponseCodeError as e:
                    logger.warning(f"[BiliBili推送] 短信验证码登录失败：{e.code}，{e.msg}")
                    logger.info("[BiliBili推送] 尝试使用二维码登录")
                    try:
                        qr_url = await bilibili_login.get_qrcode_url()
                        data = await bilibili_login.get_qrcode(qr_url)
                        if isinstance(data, bytes):
                            await app.send_friend_message(
                                BotConfig.master,
                                MessageChain(
                                    f"[BiliBili推送] 验证码登录失败：{e.msg}\n请使用 BiliBili App 扫描此二维码进行登录\n{qr_url}\n",
                                    Image(data_bytes=data),
                                ),
                            )
                        auth_data = await bilibili_login.qrcode_login(interval=5)
                        logger.success("[BiliBili推送] 二维码登录完成")
                        await app.send_friend_message(
                            BotConfig.master,
                            MessageChain("[BiliBili推送] 二维码登录完成"),
                        )
                        break
                    except ResponseCodeError as e:
                        logger.error(f"[BiliBili推送] 二维码登录失败：{e.code}，{e.msg}，如需重新登陆，请重启机器人")
                        await app.send_friend_message(
                            BotConfig.master,
                            MessageChain(
                                f"[BiliBili推送] 二维码登录失败：{e.code}，{e.msg}，如需重新登陆，请重启机器人"
                            ),
                        )
                        sys.exit(1)

        if auth_data:
            Bili_Auth.update(auth_data)
            logger.debug(await Bili_Auth.get_info())
            login_cache_file.write_text(
                json.dumps(dict(Bili_Auth), indent=2, ensure_ascii=False)
            )
            logger.info("[Bilibili推送] 登录完成")
        else:
            logger.critical("[Bilibili推送] 登录状态异常，正在退出")
            sys.exit(1)

    if BotConfig.Bilibili.use_login:
        # 初始化
        subid_list = get_all_uid()
        for _ in range(3):
            try:
                resp = await grpc_get_followed_dynamic_users(auth=Bili_Auth)
                break
            except AioRpcError as e:
                logger.error(f"[Bilibili推送] 获取关注列表失败，正在重试：{e}")
                await asyncio.sleep(5)
                continue
        else:
            error_msg = "[Bilibili推送] 获取关注列表失败，重试次数过多，正在退出"
            logger.critical(error_msg)
            await app.send_friend_message(BotConfig.master, MessageChain(error_msg))
            sys.exit(1)
        followed_list = resp.items

        if Path("data").joinpath(".lock").exists():
            if Path("data").joinpath(".lock").read_text(encoding="utf-8") != str(
                BotConfig.Bilibili.username
            ):
                logger.critical(
                    "[Bilibili推送] 检测到上次登录的账号与当前账号不一致，如需继续使用，请先手动删除数据库（data/data.db）和 data/.lock 文件"
                )
                sys.exit(1)

        elif (f := len(followed_list)) != 0:
            if cache["ignore_sub"]:
                logger.warning(f"[Bilibili推送] 该账号已关注 {f} 个用户，正在尝试自动处理")
                Path("data").joinpath(".lock").write_text(str(BotConfig.Bilibili.username))
            else:
                logger.critical(
                    f"[Bilibili推送] 该账号已关注 {f} 个用户，为避免产生问题，已停止运行，请先手动取消关注或添加启动参数 --ignore-sub，这将取关所有已关注的用户"
                )
                sys.exit(1)
        else:
            Path("data").joinpath(".lock").write_text(str(BotConfig.Bilibili.username))

        # 检测在B站关注列表但不在数据库的uid
        if followed_list:
            for uid in followed_list:
                if str(uid.uid) not in subid_list:
                    logger.warning(
                        f"[BiliBili推送] {uid.name}（{uid.uid}）在 BliBili 关注列表中，但不在数据库中，正在从 BliBili 取消关注"
                    )
                    resp = await delete_uid(uid.uid)
                    if resp:
                        await app.send_friend_message(
                            BotConfig.master,
                            MessageChain(
                                f"[BiliBili推送] {uid.name}（{uid.uid}）在 BliBili 关注列表中，但不在数据库中，正在从 BliBili 取消关注"
                            ),
                        )
                        await asyncio.sleep(1)
                        continue
                    else:
                        logger.error(
                            f"[BiliBili推送] {uid.name}（{uid.uid}）从 BliBili 取消关注失败，请检查后重启 Bot"
                        )
                        sys.exit(1)

                # 顺便检测直播状态
                if uid.live_info.status:
                    logger.info(f"[BiliBili推送] {uid.name} 已开播")
                    BOT_Status.living[str(uid.uid)] = 0

        # 检测在数据库但不在B站关注列表的uid
        for up in subid_list:
            for uid in followed_list:
                if up == str(uid.uid):
                    break
            else:
                logger.warning(f"[BiliBili推送] {up} 不在 BliBili 关注列表中，正在修复")
                resp = await relation_modify(up, 1)
                if resp["code"] != 0:
                    await delete_uid(up)
                    logger.error(f"[BiliBili推送] {up} BliBili 订阅修复失败，请检查后重启 Bot：{resp}")
                    await app.send_friend_message(
                        BotConfig.master,
                        MessageChain(f"[BiliBili推送] UP {up} BliBili 订阅修复失败，请检查后重启 Bot：{resp}"),
                    )
                    sys.exit(1)
                else:
                    logger.info(f"[BiliBili推送] {up} BliBili 订阅修复成功")
                await asyncio.sleep(1)

        logger.info(f"[BiliBili推送] 直播初始化完成，当前 {len(BOT_Status.living)} 个 UP 正在直播")

        # 动态初始化
        sub_num = len(subid_list)
        if sub_num == 0:
            await asyncio.sleep(1)
            logger.success("[BiliBili推送] 未订阅任何 UP ，初始化结束")
            BOT_Status.set_status(Status.INITIALIZED, True)
            return
        await asyncio.sleep(1)

        if resp := await grpc_get_followed_dynamics_noads():
            BOT_Status.offset["$login"] = int(resp[-1].extend.dyn_id_str)
            logger.info(f"[BiliBili推送] 动态初始化完成，offset：{BOT_Status.offset['$login']}")

            logger.success(f"[BiliBili推送] 将对 {sub_num} 个 UP 进行监控，初始化完成")
            await asyncio.sleep(1)
            await app.send_friend_message(
                BotConfig.master,
                MessageChain(f"[BiliBili推送] 将对 {sub_num} 个 UP 进行监控，初始化完成"),
            )
            BOT_Status.set_status(Status.INITIALIZED, True)
            BOT_Status.set_status(Status.STARTED, True)

    else:
        logger.info("[BiliBili推送] 未使用登录模式，正在初始化")
        subid_list = get_all_uid()
        sub_sum = len(subid_list)
        BOT_Status.offset = {}
        if sub_sum == 0:
            await asyncio.sleep(1)
            logger.success("[BiliBili推送] 未订阅任何 UP ，初始化结束")
            BOT_Status.set_status(Status.INITIALIZED, True)
            BOT_Status.set_status(Status.STARTED, True)
            return
        # 把所有账号分组，每组发送一次请求
        group_list = [
            subid_list[i : i + BotConfig.Bilibili.concurrency]
            for i in range(0, sub_sum, BotConfig.Bilibili.concurrency)
        ]
        logger.debug(f"Get {sub_sum} uid, split to {len(group_list)} groups")
        for group in group_list:
            logger.debug(f"Gathering {len(group)} uid")
            await asyncio.gather(*[init_dyn_id(uid) for uid in group])
        logger.success(f"[BiliBili推送] 将对 {sub_sum} 个 UP 进行监控，初始化完成")
        await app.send_friend_message(
            BotConfig.master,
            MessageChain(f"[BiliBili推送] 将对 {sub_sum} 个 UP 进行监控，初始化完成"),
        )
        BOT_Status.set_status(Status.INITIALIZED, True)
        BOT_Status.set_status(Status.STARTED, True)



import asyncio
import contextlib

from loguru import logger
from datetime import datetime
from graia.saya import Channel
from grpc.aio import AioRpcError
from graia.ariadne.app import Ariadne
from sentry_sdk import capture_exception
from graia.ariadne.model import MemberPerm
from graia.ariadne.message.element import AtAll
from graia.broadcast.exceptions import ExecutionStop
from graia.ariadne.message.chain import MessageChain
from graia.ariadne.connection.util import UploadMethod
from graia.scheduler.saya.schema import SchedulerSchema
from graia.scheduler.timers import every_custom_seconds
from bilireq.grpc.dynamic import grpc_get_user_dynamics
from bilireq.exceptions import ResponseCodeError, GrpcError
from graia.ariadne.exception import UnknownTarget, AccountMuted, RemoteException
from bilireq.grpc.protos.bilibili.app.dynamic.v2.dynamic_pb2 import (
    FoldType,
    DynamicItem,
    DynamicType,
    DynModuleType,
)

from ...core.context import Context
from ...core import BOT_Status, Status
from ...core.bot_config import BotConfig
from ...utils.dynamic_shot import get_dynamic_screenshot
from ...utils.up_operation import delete_group, delete_uid, set_name
from ...utils.bilibili_request import (
    get_b23_url,
    dynamic_like,
    grpc_get_dynamic_details,
    grpc_get_followed_dynamics_noads,
)
from ...core.data import (
    uid_exists,
    get_all_uid,
    is_dyn_pushed,
    get_sub_by_uid,
    insert_dynamic_push,
    is_dyn_pushed_in_group,
    insert_dyn_push_to_group,
)

channel = Channel.current()


@channel.use(SchedulerSchema(every_custom_seconds(BotConfig.Bilibili.dynamic_interval)))
async def main(app: Ariadne):
    logger.debug("[Dynamic Pusher] Dynamic Pusher running now...")
    subid_list = get_all_uid()
    sub_sum = len(subid_list)

    if not BOT_Status.is_all_statuses_true(
        Status.INITIALIZED, Status.ACCOUNT_CONNECTED, Status.STARTED, Status.SUBSCRIBE_IDLE
    ):
        logger.debug("[Dynamic Pusher] Dynamic Pusher is not init")
        await asyncio.sleep(3)
        return
    elif sub_sum == 0:
        logger.debug("[Dynamic Pusher] Dynamic Pusher is not have subids")
        await asyncio.sleep(5)
        return

    BOT_Status.set_status(Status.DYNAMIC_IDLE, False)
    BOT_Status.last_update = datetime.now().strftime("%Y-%m-%d %H:%M:%S")

    # 动态更新检测
    # 获取当前登录账号的动态列表
    logger.debug("[Dynamic] Start to get dynamic list")
    if BotConfig.Bilibili.use_login:
        try:
            dynall = await asyncio.wait_for(grpc_get_followed_dynamics_noads(), timeout=10)
        except AioRpcError as e:
            logger.error(f"[Dynamic] Get dynamic list failed: {e.details()}")
            BOT_Status.set_status(Status.DYNAMIC_IDLE, True)
            return
        except asyncio.TimeoutError:
            logger.debug("[Dynamic] Get dynamic list failed")
            BOT_Status.set_status(Status.DYNAMIC_IDLE, True)
            return

        # 判断请求是否拿到数据
        logger.debug("[Dynamic] Start to check dynamic")
        if dynall:
            logger.debug(f"[Dynamic] Get {len(dynall)} dynamics")
            new_dyn = [
                x
                for x in dynall
                if int(x.extend.dyn_id_str)
                > BOT_Status.to_dict().get("offset", int(x.extend.dyn_id_str))
                and not is_dyn_pushed(int(x.extend.dyn_id_str))
            ]
            logger.debug(f"[Dynamic] {len(new_dyn)} new dynamics")
            # 轮询动态列表
            for dyn in new_dyn:
                try:
                    up_id = str(dyn.modules[0].module_author.author.mid)
                    up_name = dyn.modules[0].module_author.author.name
                    dynid = int(dyn.extend.dyn_id_str)
                    logger.debug(f"[Dynamic] Check dynamic {dynid}, {up_name}({up_id})")
                    try:
                        if dynid <= BOT_Status.offset["$login"] or is_dyn_pushed(dynid):
                            continue
                    except ValueError:
                        continue
                    BOT_Status.set_status(Status.PUSH_IDLE, False)
                    if await push(app, dyn):
                        continue
                except ScreenshotError:
                    return
                finally:
                    BOT_Status.set_status(Status.PUSH_IDLE, True)

            # 将当前检测到的第一条动态 id 设置为最新的动态 id
            if BOT_Status.offset["$login"] > int(dynall[-1].extend.dyn_id_str):
                logger.info("[BiliBili推送] 有 UP 删除了动态")

            BOT_Status.offset["$login"] = int(dynall[-1].extend.dyn_id_str)
        else:
            logger.error("[Dynamic] Gotten dynamic is empty")
            logger.error(dynall)

    else:
        # 把uid分组，每组发送一次请求
        check_list = [
            subid_list[i : i + BotConfig.Bilibili.concurrency]
            for i in range(0, sub_sum, BotConfig.Bilibili.concurrency)
        ]
        logger.debug(f"Get {sub_sum} uid, split to {len(check_list)} groups")
        for subid_group in check_list:
            logger.debug(f"Gathering {len(subid_group)} uid: {' '.join(subid_group)}")
            await asyncio.gather(
                *[check_uid(app, uid) for uid in subid_group], return_exceptions=True
            )

    BOT_Status.last_finish = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    BOT_Status.set_status(Status.DYNAMIC_IDLE, True)
    logger.debug("[Dynamic] Updating finished")
    await asyncio.sleep(3)


async def push(app: Ariadne, dyn: DynamicItem):
    """推送动态"""

    up_id = str(dyn.modules[0].module_author.author.mid)
    up_name = dyn.modules[0].module_author.author.name
    dynid = dyn.extend.dyn_id_str
    dyn_desc = " ".join(
        [x.module_desc.text for x in dyn.modules if x.module_type == DynModuleType.module_desc]
    )

    logger.debug(f"[Dynamic] Start to push dynamic {dynid}")

    if uid_exists(up_id):
        logger.info(
            f"[BiliBili推送] {dynid} | {up_name}({up_id}) 更新了动态，共有 {len(get_sub_by_uid(up_id))} 个群订阅了该 UP"
        )
        # 判断折叠动态
        module_type_list = [i.module_type for i in dyn.modules]
        if DynModuleType.module_fold in module_type_list:
            logger.debug(f"[Dynamic] {dynid} | {up_name}({up_id}) is folded")
            fold = dyn.modules[module_type_list.index(DynModuleType.module_fold)]
            if fold.module_fold.fold_type == FoldType.FoldTypeUnite:
                fold_ids = fold.module_fold.fold_ids
                logger.debug(f"[Dynamic] {dynid} | {up_name}({up_id}) fold_ids: {fold_ids}")
                details = await grpc_get_dynamic_details(fold_ids)
                for dynamic in details.list:
                    try:
                        if is_dyn_pushed(dynamic.extend.dyn_id_str):
                            logger.debug(
                                f"[Dynamic] {dynid} | {up_name}({up_id}) is pushed, skip"
                            )
                            continue
                    except ValueError:
                        continue
                    await push(app, dynamic)

        logger.debug(f"[Dynamic] Getting screenshot of {dynid} | {up_name}({up_id})")
        shot_image = await get_dynamic_screenshot(dyn)

        if shot_image:
            logger.debug(f"[Dynamic] Get dynamic screenshot {dynid} | {up_name}({up_id})")
            dyn_img = await app.upload_image(shot_image, UploadMethod.Group)
            logger.debug(f"[Dynamic] Upload dynamic screenshot {dynid} | {up_name}({up_id})")
        else:
            logger.debug(
                f"[Dynamic] Get dynamic screenshot {dynid} | {up_name}({up_id}) failed"
            )
            err_msg = f"[BiliBili推送] {dynid} | {up_name}({up_id}) 更新了动态，截图失败"
            logger.error(err_msg)
            await app.send_friend_message(BotConfig.master, MessageChain(err_msg))
            BOT_Status.set_status(Status.DYNAMIC_IDLE, True)
            logger.debug("[Dynamic] Stop updating")
            raise ScreenshotError()

        if set_name(up_id, up_name):
            logger.debug(f"[Dynamic] Set {up_id} name to {up_name}")
        else:
            logger.debug(f"[Dynamic] Can't set {up_id} name to {up_name}")

        if dyn.card_type == DynamicType.forward:
            type_text = "转发了一条动态！"
        elif dyn.card_type == DynamicType.word:
            type_text = "发布了一条文字动态！"
        elif dyn.card_type == DynamicType.draw:
            type_text = "发布了一条图文动态！"
        elif dyn.card_type == DynamicType.article:
            type_text = "发布了一条专栏！"
        elif dyn.card_type == DynamicType.av:
            type_text = "发布了一条新视频！"
        else:
            type_text = "发布了一条动态！"

        if BotConfig.Event.push:
            await app.send_friend_message(
                BotConfig.master,
                MessageChain(
                    f"UP {up_name}（{up_id}）{type_text}\n",
                    dyn_img,
                    "\n",
                    await get_b23_url(f"https://t.bilibili.com/{dynid}"),
                ),
            )

        for data in get_sub_by_uid(up_id):
            if BotConfig.Debug.enable and int(data.group) not in BotConfig.Debug.groups:
                continue
            if data.dynamic:
                nick = f"*{up_nick} " if (up_nick := data.nick) else f"UP {up_name}（{up_id}）"
                msg = [
                    f"{nick}{type_text}\n",
                    dyn_img,
                    "\n",
                    await get_b23_url(f"https://t.bilibili.com/{dynid}"),
                ]
                if data.atall:
                    bot_perm = (await app.get_group(int(data.group))).account_perm

                    if bot_perm in [
                        MemberPerm.Administrator,
                        MemberPerm.Owner,
                    ]:
                        msg = [AtAll(), " "] + msg
                    else:
                        msg = ["@全体成员 "] + msg
                        msg.append(f"\n\n注：{BotConfig.name} 没有权限@全体成员")
                try:
                    if is_dyn_pushed_in_group(dynid, data.group):
                        logger.debug(
                            f"[Dynamic] {dynid} | {up_name}({up_id}) is pushed in group {data.group}, skip"
                        )
                        continue
                    logger.debug(
                        f"[Dynamic] Send dynamic {dynid} | {up_name}({up_id}) to {data.group}"
                    )
                    Context.push_type.set("dynamic")
                    Context.push_id.set(dynid)
                    await app.send_group_message(
                        int(data.group),
                        MessageChain(msg),
                    )
                    insert_dyn_push_to_group(dynid, data.group)
                    await asyncio.sleep(5)
                except (UnknownTarget, ValueError):
                    logger.warning(
                        f"[BiliBili推送] {dynid} | {up_name}({up_id}) 推送失败，找不到该群 {data.group}，正在取消订阅"
                    )
                    delete = await delete_group(data.group)
                    logger.warning(f"[BiliBili推送] 已删除群 {data.group} 订阅的 {len(delete)} 个 UP")
                    with contextlib.suppress(UnknownTarget, ValueError):
                        await app.quit_group(int(data.group))
                except AccountMuted:
                    group = await app.get_group(int(data.group))
                    group = f"{group.name}（{group.id}）" if group else data.group
                    logger.warning(
                        f"[BiliBili推送] {dynid} | {up_name}({up_id}) 推送失败，账号在 {group} 被禁言"
                    )
                except RemoteException as e:
                    if "LIMITED_MESSAGING" in str(e):
                        logger.error(
                            f"[BiliBili推送] {dynid} | {up_name}({up_id}) 推送失败，Bot 被限制发送群聊消息"
                        )
                        await app.send_friend_message(
                            BotConfig.master,
                            MessageChain(
                                "Bot 被限制发送群聊消息（46 代码），问题原因可能是账号被多次举报或被服务器认为不安全. "
                                "若账号在官方客户端也无法发出消息, 可尝试用手机 QQ 登录后访问 "
                                "https://accounts.qq.com/safe/message/unlock?lock_info=5_5 解冻。"
                                "请尽快处理后发送 /init 重新开启推送进程"
                            ),
                        )
                        BOT_Status.set_status(Status.DYNAMIC_IDLE, True)
                        BOT_Status.set_status(Status.INITIALIZED, False)
                        raise ExecutionStop() from e
                    elif "resultType=110" in str(e):  # 110: 可能为群被封
                        logger.warning(
                            f"[BiliBili推送] {dynid} | {up_name}({up_id}) 推送失败，Bot 因未知原因被移出群聊"
                        )
                        delete = await delete_group(data.group)
                        logger.warning(f"[BiliBili推送] 已删除群 {data.group} 订阅的 {len(delete)} 个 UP")
                        with contextlib.suppress(UnknownTarget):
                            await app.quit_group(int(data.group))
                    elif "reason=AT_ALL_LIMITED" in str(e):
                        logger.warning(
                            f"[BiliBili推送] {dynid} | {up_name}({up_id}) 推送失败，Bot 在该群 @全体成员 次数已达上限"
                        )
                        await app.send_group_message(int(data.group), MessageChain(msg[1:]))
                        await asyncio.sleep(1)
                    else:
                        capture_exception()
                        logger.exception(f"[BiliBili推送] {dynid} | {up_name}({up_id}) 推送失败，未知错误")
                except Exception:  # noqa
                    capture_exception()
                    logger.exception(f"[BiliBili推送] {dynid} | {up_name}({up_id}) 推送失败，未知错误")
        if BotConfig.Bilibili.use_login:
            try:
                await dynamic_like(dynid)
                logger.info(f"[BiliBili推送] {dynid} | {up_name}({up_id}) 动态点赞成功")
            except ResponseCodeError as e:
                logger.error(f"[BiliBili推送] {dynid} | {up_name}({up_id}) 动态点赞失败：{e}")
        insert_dynamic_push(
            up_id,
            up_name,
            dynid,
            dyn.card_type,
            dyn_desc,
            len(get_sub_by_uid(up_id)),
        )
    else:
        logger.warning(
            f"[BiliBili推送] {dynid} | {up_name}({up_id}) 没有找到订阅 UP {up_name}（{up_id}）的群，已退订！"
        )
        await delete_uid(up_id)


async def check_uid(app: Ariadne, uid):
    try:
        resp = await asyncio.wait_for(grpc_get_user_dynamics(int(uid)), timeout=10)
    except asyncio.TimeoutError:
        logger.warning(f"[BiliBili推送] {uid} 获取动态超时！")
        return
    except (GrpcError, AioRpcError) as e:
        logger.error(
            f"[BiliBili推送] {uid} 获取动态失败：[{e.code}] {e.details() if isinstance(e, AioRpcError) else e.msg}"
        )
        return
    except Exception as e:  # noqa
        capture_exception(e)
        raise e
    if resp:
        resp = [
            x
            for x in resp.list
            if int(x.extend.dyn_id_str) > BOT_Status.offset.get(uid, int(x.extend.dyn_id_str))  # type: ignore
        ]
        resp.reverse()
        for dyn in resp:
            if dyn.modules[0].module_author.is_top:
                continue
            try:
                up_id = str(dyn.modules[0].module_author.author.mid)
                up_name = dyn.modules[0].module_author.author.name
                dynid = int(dyn.extend.dyn_id_str)
                logger.debug(f"[Dynamic] Check dynamic {dynid} | {up_name}({up_id})")
                try:
                    if dynid <= BOT_Status.offset[up_id] or is_dyn_pushed(dynid):
                        return
                except ValueError:
                    BOT_Status.offset[up_id] = dynid
                    return
                await push(app, dyn)

                BOT_Status.offset[up_id] = dynid

            except ScreenshotError:
                return


class ScreenshotError(Exception):
    pass


@channel.use(SchedulerSchema(every_custom_seconds(2)))
async def debug():
    logger.debug(BOT_Status.to_dict())



from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.model import Friend
from graia.ariadne.message.chain import MessageChain
from graia.ariadne.event.message import FriendMessage
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.message.parser.twilight import (
    Twilight,
    ArgResult,
    RegexMatch,
    ArgumentMatch,
)

from ....core.control import Permission
from ....core.subgroup_config import SubGroup

channel = Channel.current()

REMOVE_ERROR_MSG = """删除失败, 请正确输入!
请按照 [指令 --name 组名 --uid UID] 格式进行添加

指令：
从订阅组(移除|删除)(主播|[uU][pP])

参数：
--name 订阅组名称
--uid up主的uid

例:
从订阅组删除主播 --name XXX --uid 123456
从订阅组移除up --name XXX --uid 123456
"""


@channel.use(
    ListenerSchema(
        listening_events=[FriendMessage],
        inline_dispatchers=[
            Twilight(
                [
                    RegexMatch(r"从订阅组(删除|移除)(主播|[uU][pP])"),
                    "group_name" @ ArgumentMatch("--name", optional=True, nargs="*", type=str),
                    "uid" @ ArgumentMatch("--uid", optional=True, nargs="*", type=int),
                ],
            )
        ],
    )
)
async def main(app: Ariadne, friend: Friend, group_name: ArgResult, uid: ArgResult):
    Permission.manual(friend, Permission.MASTER)

    if group_name.matched and uid.matched:
        group_name = group_name.result[0]
        uid_ = uid.result[0]
        sg = SubGroup(group_name)
        if sg.is_in_group_names():
            if sg.remove_from_subGroup_ups(uid_):
                await app.send_friend_message(
                    friend,
                    MessageChain(f"从 [{group_name}] 订阅组中删除 [uid: {uid_}] 成功"),
                )
            else:
                await app.send_friend_message(
                    friend, MessageChain(f"删除失败, 订阅组 [{group_name}] 中并无订阅的up主!")
                )
        else:
            await app.send_friend_message(friend, MessageChain("删除的订阅组名称不存在!"))
    else:
        await app.send_friend_message(friend, MessageChain(REMOVE_ERROR_MSG))



import httpx
import asyncio

from loguru import logger
from typing import Optional
from httpx import TimeoutException
from sentry_sdk import capture_exception

from ..core.bot_config import BotConfig
from ..model.exception import AbortError
from ..model.bcut_asr import ResultStateEnum

from .bcut_asr import BcutASR
from .bilibili_request import get_player, hc
from .bilibili_request import grpc_get_playview


async def get_subtitle_url(aid: int, cid: int) -> Optional[str]:
    video_player = await get_player(aid, cid)
    subtitles_raw: list[dict] = video_player["subtitle"]["subtitles"]
    logger.debug(subtitles_raw)

    if not subtitles_raw:
        return

    logger.debug(subtitles_raw)
    ai_subtitles = {}
    manual_subtitles = {}
    preferred_subs = ["zh-Hans", "en-US"]

    for subtitle in subtitles_raw:
        if "自动生成" in subtitle["lan_doc"]:
            ai_subtitles[subtitle["lan"]] = subtitle["subtitle_url"]
        else:
            manual_subtitles[subtitle["lan"]] = subtitle["subtitle_url"]

    if not manual_subtitles:
        return next(iter(ai_subtitles.values()))

    for sub in preferred_subs:
        if sub in manual_subtitles:
            return manual_subtitles[sub]

    return next(iter(manual_subtitles.values()))


async def get_subtitle(aid: int, cid: int) -> list[str]:
    subtitle_url = await get_subtitle_url(aid, cid)
    if subtitle_url:
        logger.debug(subtitle_url)
        subtitle = await hc.get(f"https:{subtitle_url}")
        if subtitle.status_code != 200:
            logger.warning(f"字幕获取失败：{aid} {cid}，状态码：{subtitle.status_code}，内容：{subtitle.text}")
            raise AbortError("字幕下载失败")
        logger.info(f"字幕获取成功：{aid} {cid}")
    elif BotConfig.Bilibili.use_bcut_asr:
        logger.info(f"字幕获取失败，尝试使用 BCut-ASR：{aid} {cid}")
        playview = await grpc_get_playview(aid, cid)
        if not playview.video_info.dash_audio:
            raise AbortError("视频无音频流")
        async with httpx.AsyncClient(
            headers={
                "user-agent": "Bilibili Freedoooooom/MarkII",
            },
        ) as client:
            audio_resp = await client.get(
                playview.video_info.dash_audio[-1].backup_url[0]
                if playview.video_info.dash_audio[-1].backup_url
                else playview.video_info.dash_audio[-1].baseUrl,
            )
        audio_resp.raise_for_status()
        audio = audio_resp.content
        try:
            asr = await get_bcut_asr(audio)
        except TimeoutException as e:
            logger.error("BCut-ASR 连接超时")
            raise AbortError("BCut-ASR 连接超时") from e
        except Exception as e:
            logger.exception("BCut-ASR 识别失败")
            capture_exception()
            raise AbortError("BCut-ASR 识别失败") from e
        return [x.transcript for x in asr]
    else:
        raise AbortError("未找到字幕且未开启 AI 语音识别")
    return [x["content"] for x in subtitle.json()["body"]]


async def get_bcut_asr(file_bytes: bytes):
    bcut = BcutASR()
    bcut.set_data(raw_data=file_bytes, data_fmt="m4a")
    await bcut.upload()
    await bcut.create_task()
    while True:
        result = await bcut.result()
        if result.state == ResultStateEnum.COMPLETE:
            return result.parse()
        elif result.state in [ResultStateEnum.RUNING, ResultStateEnum.STOP]:
            logger.info(f"[BCut-ASR] 任务 {result.task_id} 正在进行中 - {result.state}...")
            await asyncio.sleep(2)
        elif result.state == ResultStateEnum.ERROR:
            logger.error(f"[BCut-ASR] 任务 {result.task_id} 发生错误！")
            raise AbortError("语音识别出错")



from typing import Optional
from pydantic import BaseModel


class CaptchaData(BaseModel):
    captcha_id: str
    points: list[list[int]]
    rectangles: list[list[int]]
    yolo_data: list[list[int]]
    time: int


class CaptchaResponse(BaseModel):
    code: int
    message: str
    data: Optional[CaptchaData]



from loguru import logger
from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.exception import UnknownTarget
from graia.ariadne.message.chain import MessageChain
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.event.mirai import BotInvitedJoinGroupRequestEvent

from ...core.bot_config import BotConfig
from ...core.group_config import GroupPermission

channel = Channel.current()


@channel.use(ListenerSchema(listening_events=[BotInvitedJoinGroupRequestEvent]))
async def main(app: Ariadne, event: BotInvitedJoinGroupRequestEvent):
    """
    收到入群事件
    """
    gp = GroupPermission(event.source_group)
    msg = [
        f"收到入群邀请事件：{event.group_name}（{event.source_group}）\n",
        f"邀请来源：{event.nickname}（{event.supplicant}）\n",
    ]
    if gp.can_join():
        await event.accept()
        for admin in BotConfig.admins:
            try:
                await app.send_friend_message(
                    admin,
                    MessageChain(
                        msg,
                        "已自动同意加入",
                    ),
                )
            except (UnknownTarget, ValueError):
                logger.warning(f"由于未添加 {admin} 为好友，无法发送通知")
    else:
        await event.reject()
        for admin in BotConfig.admins:
            try:
                await app.send_friend_message(
                    admin,
                    MessageChain(
                        msg,
                        "该群不在白名单中，已拒绝加入",
                    ),
                )
                await app.send_friend_message(event.supplicant, MessageChain("该群不在白名单中，已拒绝加入"))
            except (UnknownTarget, ValueError):
                logger.warning(f"由于未添加 {admin} 为好友，无法发送通知")



from enum import Flag, auto
from typing import Optional
from bilireq.auth import Auth


class Status(Flag):
    STARTED = auto()
    """程序启动"""
    INITIALIZED = auto()
    """已初始化"""
    PUSH_IDLE = auto()
    """推送进程空闲"""
    SUBSCRIBE_IDLE = auto()
    """订阅处理进程空闲"""
    DYNAMIC_IDLE = auto()
    """动态更新进程空闲"""
    LIVE_IDLE = auto()
    """直播更新进程空闲"""
    ACCOUNT_CONNECTED = auto()
    """账号已连接"""

    ALL_STATUS = (
        STARTED
        | INITIALIZED
        | PUSH_IDLE
        | SUBSCRIBE_IDLE
        | DYNAMIC_IDLE
        | LIVE_IDLE
        | ACCOUNT_CONNECTED
    )


class BBotStatus:
    def __init__(self):
        self.status = Status(0)
        self.living: dict[str, int] = {}
        self.offset: dict[str, int] = {}
        self.last_update: Optional[str] = None
        self.last_finish: Optional[str] = None

        self.set_status(Status.DYNAMIC_IDLE, True)
        self.set_status(Status.LIVE_IDLE, True)
        self.set_status(Status.PUSH_IDLE, True)
        self.set_status(Status.SUBSCRIBE_IDLE, True)

    def set_status(self, status, value: bool):
        if value:
            self.status |= status
        else:
            self.status &= ~status

    def check_status(self, status) -> bool:
        return self.status & status == status

    def is_all_statuses_true(self, *statuses):
        all_status = Status(0)
        for status in statuses:
            all_status |= status
        return self.check_status(all_status)

    def is_all_status_true(self):
        return self.check_status(Status.ALL_STATUS)

    def to_dict(self):
        status_dict = {
            member.name: self.check_status(member) for member in Status.__members__.values()
        }
        return {
            "status": status_dict,
            "living": self.living,
            "offset": self.offset,
        }


BOT_Status = BBotStatus()
Bili_Auth = Auth()
cache = {}



from graia.saya import Channel
from graia.ariadne.event.message import GroupMessage
from graia.saya.builtins.broadcast.schema import ListenerSchema

from ...core.data import add_talk_count

channel = Channel.current()


@channel.use(ListenerSchema(listening_events=[GroupMessage]))
async def main():

    add_talk_count()



from graia.ariadne.app import Ariadne
from fastapi import APIRouter, Depends, status

from .auth import verify_token, can_manage_groups, can_manage_group
from ....model.fastapi import Info, GroupListResponse, MemberListResponse, MemberItem

router = APIRouter(tags=["Users"])


@router.get("/get_groups", response_model=GroupListResponse, summary="获取当前可管理的群列表")
async def get_groups(info: Info = Depends(verify_token)):
    """获取群列表"""
    return GroupListResponse(data=await can_manage_groups(info))


@router.get("/get_group_members/{group}", response_model=MemberListResponse, summary="获取群成员列表")
async def get_group_members(group: int, info: Info = Depends(verify_token)):
    """获取群成员列表"""
    app = Ariadne.current()
    return (
        MemberListResponse(
            data=[
                MemberItem(
                    id=x.id,
                    name=x.name,
                    permission=x.permission,
                    special_title=x.special_title,
                    join_timestamp=x.join_timestamp,
                    last_speak_timestamp=x.last_speak_timestamp,
                    mute=x.mute_time or 0,
                )
                for x in await app.get_member_list(group)
            ]
        )
        if can_manage_group(info, group)
        else GroupListResponse(
            code=status.HTTP_403_FORBIDDEN, message="Permission denied", data=[]
        )
    )



import httpx

from typing import Union
from loguru import logger
from bilireq.utils import post, get
from bilireq.grpc.utils import grpc_request
from bilireq.grpc.dynamic import grpc_get_followed_dynamics
from bilireq.grpc.protos.bilibili.app.view.v1.view_pb2_grpc import ViewStub
from bilireq.grpc.protos.bilibili.app.view.v1.view_pb2 import ViewReq, ViewReply
from bilireq.grpc.protos.bilibili.community.service.dm.v1.dm_pb2_grpc import DMStub
from bilireq.grpc.protos.bilibili.app.dynamic.v2.dynamic_pb2_grpc import DynamicStub
from bilireq.grpc.protos.bilibili.app.playurl.v1.playurl_pb2_grpc import PlayURLStub
from bilireq.grpc.protos.bilibili.app.playurl.v1.playurl_pb2 import PlayViewReq, PlayViewReply
from bilireq.grpc.protos.bilibili.community.service.dm.v1.dm_pb2 import DmViewReq, DmViewReply
from bilireq.grpc.protos.bilibili.app.dynamic.v2.dynamic_pb2 import (
    DynamicType,
    DynDetailsReq,
    DynDetailsReply,
)

from ..core import Bili_Auth

hc = httpx.AsyncClient(
    headers={
        "User-Agent": (
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
            "(KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36 Edg/112.0.1722.39"
        )
    }
)


async def relation_modify(uid: Union[str, int], act: int):
    """
    修改关系

    Args:
        uid: 用户ID
        act: 操作
    Act:
        1:	 关注
        2:	 取关
        3:	 悄悄关注
        4:	 取消悄悄关注
        5:	 拉黑
        6:	 取消拉黑
        7:	 踢出粉丝团
    """
    url = "https://api.bilibili.com/x/relation/modify"
    data = {
        "act": act,
        "fid": int(uid),
    }
    return await post(url, params=data, auth=Bili_Auth, raw=True)


async def dynamic_like(dynid):
    """
    动态点赞

    Args:
        dynid: 动态ID
    """
    url = "https://api.vc.bilibili.com/dynamic_like/v1/dynamic_like/thumb"
    data = {"dynamic_id": str(dynid), "up": 1}
    return await post(url, data=data, auth=Bili_Auth)


async def get_b23_url(burl: str) -> str:
    """
    b23 链接转换

    Args:
        burl: 需要转换的 BiliBili 链接
    """
    url = "https://api.bilibili.com/x/share/click"
    data = {
        "build": 6700300,
        "buvid": 0,
        "oid": burl,
        "platform": "android",
        "share_channel": "COPY",
        "share_id": "public.webview.0.0.pv",
        "share_mode": 3,
    }
    resp = await post(url, data=data)
    logger.debug(resp)
    return resp["content"]


async def search_user(keyword: str):
    """
    搜索用户
    """
    url = "https://app.bilibili.com/x/v2/search/type"
    data = {"build": "6840300", "keyword": keyword, "type": "2", "ps": 5}

    resp = await get(url, params=data)
    logger.debug(resp)
    return resp


async def get_user_space_info(uid: int):
    """
    获取用户空间信息
    """
    url = "https://app.bilibili.com/x/v2/space"
    params = {
        "vmid": uid,
        "build": 6840300,
        "ps": 1,
    }
    return await get(url, params=params)


async def get_player(aid: int, cid: int):
    """
    获取视频播放器信息
    """
    url = "https://api.bilibili.com/x/player/v2"
    params = {
        "aid": aid,
        "cid": cid,
    }
    return await get(url, params=params, auth=Bili_Auth)


async def grpc_get_followed_dynamics_noads():
    resp = await grpc_get_followed_dynamics(auth=Bili_Auth)
    exclude_list = [
        DynamicType.ad,
        DynamicType.live,
        DynamicType.live_rcmd,
        DynamicType.banner,
    ]
    dynamic_list = [dyn for dyn in resp.dynamic_list.list if dyn.card_type not in exclude_list]
    dynamic_list.reverse()
    return dynamic_list


@grpc_request
async def grpc_get_dynamic_details(dynamic_ids: str, **kwargs) -> DynDetailsReply:
    stub = DynamicStub(kwargs.pop("_channel"))
    req = DynDetailsReq(dynamic_ids=dynamic_ids)
    return await stub.DynDetails(req, **kwargs)


@grpc_request
async def grpc_get_view_info(aid: int = 0, bvid: str = "", **kwargs) -> ViewReply:
    stub = ViewStub(kwargs.pop("_channel"))
    if aid:
        req = ViewReq(aid=aid)
    elif bvid:
        req = ViewReq(bvid=bvid)
    else:
        raise ValueError("aid or bvid must be provided")
    return await stub.View(req, **kwargs)


@grpc_request
async def grpc_get_dmview(pid: int, oid: int, type: int = 1, **kwargs) -> DmViewReply:
    stub = DMStub(kwargs.pop("_channel"))
    req = DmViewReq(pid=pid, oid=oid, type=type)
    return await stub.DmView(req, **kwargs)


@grpc_request
async def grpc_get_playview(aid: int, cid: int, **kwargs) -> PlayViewReply:
    stub = PlayURLStub(kwargs.pop("_channel"))
    req = PlayViewReq(aid=aid, cid=cid, qn=128, fnval=464)
    return await stub.PlayView(req, **kwargs)



import asyncio
import base64
import re
from datetime import datetime
from io import BytesIO
from pathlib import Path
from typing import List, Optional, Union

import qrcode
from bilireq.exceptions import ResponseCodeError
from bilireq.grpc.protos.bilibili.app.view.v1.view_pb2 import ViewReply
from httpx import AsyncClient
from PIL import Image, ImageDraw, ImageFont
from qrcode.image.pil import PilImage

from .bilibili_request import get_user_space_info, hc
from .fonts_provider import get_font_sync
from .strings import get_cut_str, num_fmt

font_semibold = str(get_font_sync("sarasa-mono-sc-semibold.ttf"))
font_bold = str(get_font_sync("sarasa-mono-sc-bold.ttf"))
font_vanfont = str(get_font_sync("vanfont.ttf"))


class UP:
    def __init__(
        self,
        name: str,
        face: Union[bytes, BytesIO],
        level: int,
        fans: str,
        video_counts: int,
        title: str = "UP主",
        name_color: str = "black",
        official_verify: int = -1,
    ) -> None:
        self.name: str = name
        """UP名"""
        self.face: BytesIO = face if isinstance(face, BytesIO) else BytesIO(face)
        """UP头像"""
        self.level: int = level
        """UP等级"""
        self.name_color: str = name_color or "black"
        """UP名字颜色"""
        self.fans: str = fans
        """粉丝数"""
        self.video_counts: int = video_counts
        """视频投稿数量"""
        self.title: str = title
        """合作视频中的角色"""
        self.official_verify: int = official_verify
        """小闪电认证：-1 为无，0 为个人，1 为企业"""

    @classmethod
    async def from_id(cls, client: AsyncClient, mid: int, name: str, title: str):
        try:
            up_data = await get_user_space_info(mid)
        except ResponseCodeError as e:
            if e.code in [-404, 404]:
                up_data = None
            else:
                raise e
        bg_color = up_data["card"]["vip"]["label"]["bg_color"] if up_data else "black"
        level = up_data["card"]["level_info"]["current_level"] if up_data else 0
        video_counts = up_data["archive"]["count"] if up_data else 0
        face_url = (
            up_data["card"]["face"]
            if up_data
            else "https://i0.hdslb.com/bfs/face/member/noface.jpg"
        )
        official_verify = up_data["card"]["official_verify"]["type"] if up_data else -1
        face_req = await client.get(face_url)
        if face_req.status_code == 404:
            face_req = await client.get(f"{face_url}@160w_160h_1c_1s.webp")
        face = face_req.content
        return cls(
            name=name,
            name_color=bg_color,
            level=level,
            face=face,
            fans=num_fmt(up_data["card"]["fans"]) if up_data else "N/A",
            video_counts=video_counts,
            title=title,
            official_verify=official_verify,
        )


class BiliVideoImage:
    def __init__(
        self,
        cover: Union[bytes, BytesIO],
        duration: int,
        type_name: str,
        title: str,
        view: str,
        danmaku: str,
        favorite: str,
        coin: str,
        like: str,
        reply: str,
        share: str,
        pubdate: datetime,
        uploaders: List[UP],
        b23_url: str,
        aid: str,
        desc: Optional[str] = None,
    ):
        self.cover: BytesIO = cover if isinstance(cover, BytesIO) else BytesIO(cover)
        """视频封面"""
        minutes, self.seconds = divmod(duration, 60)
        self.hours, self.minutes = divmod(minutes, 60)
        self.type_name: str = type_name
        """视频分区"""
        self.title: str = title
        """视频标题"""
        self.desc: str = desc or "该视频没有简介"
        """视频简介"""
        self.view: str = view
        """播放量"""
        self.danmaku: str = danmaku
        """弹幕数"""
        self.favorite: str = favorite
        """收藏"""
        self.coin: str = coin
        """投币"""
        self.like: str = like
        """点赞"""
        self.reply: str = reply
        """评论"""
        self.share: str = share
        """分享"""
        self.pubdate: datetime = pubdate
        """发布时间"""
        self.uploaders: List[UP] = uploaders
        """up主列表"""
        self.b23_url: str = b23_url
        """b23短链"""
        self.aid: str = aid
        """av号"""

    @classmethod
    async def from_view_rely(cls, video_view: ViewReply, b23_url: str) -> "BiliVideoImage":
        video_info = (
            video_view.activity_season if video_view.activity_season.arc.aid else video_view
        )
        # 获取封面
        client = hc
        client.headers.update(
            {
                "User-Agent": (
                    "Mozilla/5.0 (Windows NT 10.0; Win64; x64)"
                    "AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.149 Safari/537.36"
                ),
                "origin": "https://www.bilibili.com",
                "referer": f"https://www.bilibili.com/video/{video_info.bvid}",
            }
        )
        # 封面
        cover_bytes = (await client.get(video_info.arc.pic)).content
        # up列表
        ups = []
        # 合作视频(UP在第一个)
        if video_info.staff:
            for staff in video_info.staff:
                ups.append(await UP.from_id(client, staff.mid, staff.name, staff.title))
        # 单人视频
        else:
            ups.append(
                await UP.from_id(
                    client, video_info.arc.author.mid, video_info.arc.author.name, "UP主"
                )
            )

        return cls(
            cover=cover_bytes,
            duration=video_info.arc.duration,
            type_name=video_info.arc.type_name,
            title=video_info.arc.title,
            desc=video_info.arc.desc,
            view=num_fmt(video_info.arc.stat.view),
            danmaku=num_fmt(video_info.arc.stat.danmaku),
            favorite=num_fmt(video_info.arc.stat.fav),
            coin=num_fmt(video_info.arc.stat.coin),
            like=num_fmt(video_info.arc.stat.like),
            reply=num_fmt(video_info.arc.stat.reply),
            share=num_fmt(video_info.arc.stat.share),
            pubdate=datetime.fromtimestamp(video_info.arc.pubdate),
            uploaders=ups,
            b23_url=b23_url,
            aid=f"av{video_info.arc.aid}",
        )

    @staticmethod
    def get_up_level_code(level: int):
        if level == 0:
            up_level = "\uE6CB"
            level_color = (191, 191, 191)
        elif level == 1:
            up_level = "\uE6CC"
            level_color = (191, 191, 191)
        elif level == 2:
            up_level = "\uE6CD"
            level_color = (149, 221, 178)
        elif level == 3:
            up_level = "\uE6CE"
            level_color = (146, 209, 229)
        elif level == 4:
            up_level = "\uE6CF"
            level_color = (255, 179, 124)
        elif level == 5:
            up_level = "\uE6D0"
            level_color = (255, 108, 0)
        else:
            up_level = "\uE6D1"
            level_color = (255, 0, 0)
        return up_level, level_color

    async def render(self, style: str = "style_blue"):
        if style == "bbot_default":
            loop = asyncio.get_running_loop()
            return await loop.run_in_executor(None, self.style_bbot_default)  # type: ignore
        if style == "style_blue":
            return await self.style_blue()

    def style_bbot_default(self):
        bg_y = 0
        # ===  封面 ===
        ## 封面
        cover = Image.open(self.cover)
        pic = cover.resize((560, 350))
        pic_time_box = Image.new("RGBA", (560, 50), (0, 0, 0, 150))
        pic.paste(pic_time_box, (0, 300), pic_time_box)
        bg_y += 350 + 20
        ## 时长
        video_time = f"{self.hours:02d}:{self.minutes:02d}:{self.seconds:02d}"
        time_font = ImageFont.truetype(font_bold, 30)
        draw = ImageDraw.Draw(pic)
        draw.text((10, 305), video_time, "white", time_font)
        ## 分区
        tname = self.type_name
        tname_x, _ = time_font.getsize(tname)
        draw.text((560 - tname_x - 10, 305), tname, "white", time_font)

        # === 标题 ===
        title_font = ImageFont.truetype(font_bold, 25)
        title_cut_str = "\n".join(get_cut_str(self.title, 40))
        _, title_text_y = title_font.getsize_multiline(title_cut_str)
        title_bg = Image.new("RGB", (560, title_text_y + 23), "#F5F5F7")
        draw = ImageDraw.Draw(title_bg)
        draw.text((15, 10), title_cut_str, "black", title_font)
        title_bg_y = title_bg.size[1]
        bg_y += title_bg_y

        # === 简介 ===
        dynamic = self.desc
        dynamic_font = ImageFont.truetype(font_semibold, 18)
        dynamic_cut_str = "\n".join(get_cut_str(dynamic, 58))
        _, dynamic_text_y = dynamic_font.getsize_multiline(dynamic_cut_str)
        dynamic_bg = Image.new("RGB", (560, dynamic_text_y + 24), "#F5F5F7")
        draw = ImageDraw.Draw(dynamic_bg)
        draw.rectangle((0, 0, 580, dynamic_text_y + 24), "#E1E1E5")
        draw.text((10, 10), dynamic_cut_str, "#474747", dynamic_font)
        dynamic_bg_y = dynamic_bg.size[1]
        bg_y += dynamic_bg_y

        # === 视频数据 ===
        icon_font = ImageFont.truetype(font_vanfont, 46)
        icon_color = (247, 145, 185)
        info_font = ImageFont.truetype(font_bold, 26)
        info_bg = Image.new("RGB", (560, 170), "#F5F5F7")
        draw = ImageDraw.Draw(info_bg)
        ## 点赞
        draw.text((5 + 10, 20), "\uE6E0", icon_color, icon_font)
        draw.text((5 + 64, 27), self.like, "#474747", info_font)
        ## 投币
        draw.text((5 + 10 + 180, 20), "\uE6E4", icon_color, icon_font)
        draw.text((5 + 64 + 180, 27), self.coin, "#474747", info_font)
        ## 收藏
        draw.text((5 + 10 + 180 + 180, 20), "\uE6E1", icon_color, icon_font)
        draw.text((5 + 64 + 180 + 180, 27), self.favorite, "#474747", info_font)
        ## 播放
        draw.text((5 + 100, 93), "\uE6E6", icon_color, icon_font)
        draw.text((5 + 154, 100), self.view, "#474747", info_font)
        ## 弹幕
        draw.text((5 + 100 + 210, 93), "\uE6E7", icon_color, icon_font)
        draw.text((5 + 154 + 210, 100), self.danmaku, "#474747", info_font)
        #
        info_bg_y = info_bg.size[1]
        bg_y += info_bg_y

        # UP主
        # 等级 0-4 \uE6CB-F 5-6\uE6D0-1
        # UP \uE723
        up_num = len(self.uploaders)
        up_bg = Image.new("RGB", (560, 20 + (up_num * 120) + 20), "#F5F5F7")
        draw = ImageDraw.Draw(up_bg)
        face_size = (80, 80)
        mask = Image.new("RGBA", face_size, color=(0, 0, 0, 0))
        mask_draw = ImageDraw.Draw(mask)
        mask_draw.ellipse((0, 0, face_size[0], face_size[1]), fill=(0, 0, 0, 255))
        name_font = ImageFont.truetype(font_bold, 24)
        up_title_font = ImageFont.truetype(font_bold, 20)
        follower_font = ImageFont.truetype(font_semibold, 22)
        for i, up in enumerate(self.uploaders):
            up_level, level_color = self.get_up_level_code(up.level)
            # 头像
            face = Image.open(up.face)
            face.convert("RGB")
            face = face.resize(face_size)
            up_bg.paste(face, (20, 20 + (i * 120)), mask)
            # 名字
            draw.text(
                (160, 25 + (i * 120)),
                up.name,
                up.name_color,
                name_font,
            )
            name_size_x, _ = name_font.getsize(up.name)
            # 等级
            draw.text(
                (160 + name_size_x + 10, 16 + (i * 120)), up_level, level_color, icon_font
            )
            # 身份
            up_title_size_x, up_title_size_y = up_title_font.getsize(up.title)
            draw.rectangle(
                (
                    60,
                    10 + (i * 120),
                    73 + up_title_size_x,
                    18 + (i * 120) + up_title_size_y,
                ),
                "white",
                icon_color,
                3,
            )
            draw.text((67, 13 + (i * 120)), up.title, icon_color, up_title_font)
            # 粉丝量
            draw.text((162, 66 + (i * 120)), f"粉丝 {up.fans}", "#474747", follower_font)

        # === UP ===
        up_bg_y = up_bg.size[1]
        bg_y += up_bg_y - 18
        # 底部栏
        banner_bg = Image.new("RGB", (600, 170), icon_color)
        draw = ImageDraw.Draw(banner_bg)
        # 二维码
        qr = qrcode.QRCode(border=1)
        qr.add_data(self.b23_url)
        qr_image = qr.make_image(PilImage, fill_color=icon_color, back_color="#F5F5F7")
        qr_image = qr_image.resize((140, 140))
        banner_bg.paste(qr_image, (50, 10))
        # Logo
        # LOGO \uE725
        logo_font = ImageFont.truetype(font_vanfont, 100)
        draw.text((300, 28), "\uE725", "#F5F5F7", logo_font)
        bg_y += 170

        # === 汇总 ===
        video = Image.new("RGB", (600, bg_y + 40), "#F5F5F7")
        video.paste(pic, (20, 20))
        video.paste(title_bg, (20, 390))
        video.paste(dynamic_bg, (20, 390 + title_bg_y + 20))
        video.paste(info_bg, (20, 390 + title_bg_y + 20 + dynamic_bg_y + 20))
        video.paste(up_bg, (20, 390 + title_bg_y + 20 + dynamic_bg_y + 10 + info_bg_y))
        video.paste(
            banner_bg,
            (0, 390 + title_bg_y + 20 + dynamic_bg_y + 10 + info_bg_y + up_bg_y - 18),
        )
        image = BytesIO()
        video.save(image, "JPEG")

        return image.getvalue()

    async def style_blue(self):
        import jinja2
        from graia.ariadne.app import Ariadne
        from graiax.playwright.interface import PlaywrightContext

        from .browser_shot import fill_font

        app = Ariadne.current()
        browser_context = app.launch_manager.get_interface(PlaywrightContext).context

        style_bule = Path(__file__).parent.parent.joinpath("static", "style_blue")
        video_time = (
            f"{self.hours:02d}:{self.minutes:02d}:{self.seconds:02d}"
            if self.hours
            else f"{self.minutes:02d}:{self.seconds:02d}"
        )
        qr = qrcode.QRCode(border=1)
        qr.add_data(self.b23_url)
        qr_image = BytesIO()
        qr.make_image(PilImage).save(qr_image, "JPEG")

        ups = []
        for up in self.uploaders:
            level, level_color = self.get_up_level_code(up.level)
            info = {
                "avatar_image": f"data:image/png;base64,{base64.b64encode(up.face.getvalue()).decode()}",
                "name": up.name,
                "level": level,
                "name_color": up.name_color,
                "level_color": f"rgba{level_color}",
                "fans_count": up.fans,
                "video_count": up.video_counts,
                "icon": up.official_verify,
                "condition": up.title,
            }
            ups.append(info)

        template_env = jinja2.Environment(
            loader=jinja2.FileSystemLoader(style_bule),
            enable_async=True,
        )
        template_path = (
            f"file:///{style_bule.joinpath('video-details.html').absolute()}".replace(
                "////", "///"
            )
        )
        template = template_env.get_template("video-details.html")
        html = await template.render_async(
            **{
                "cover_image": f"data:image/png;base64,{base64.b64encode(self.cover.getvalue()).decode()}",
                "video_category": self.type_name,
                "video_duration": video_time,
                "up_infos": ups,
                "video_title": self.title,
                "view_count": self.view,
                "dm_count": self.danmaku,
                "reply_count": self.reply,
                "upload_date": self.pubdate.strftime("%Y-%m-%d"),
                "av_number": self.aid,
                "video_summary": self.desc.replace("\n", "<br>"),
                "like_count": self.like,
                "coin_count": self.coin,
                "fav_count": self.favorite,
                "share_count": self.share,
                "qr_code_image": f"data:image/png;base64,{base64.b64encode(qr_image.getvalue()).decode()}",
            }
        )

        async with await browser_context.new_page() as page:
            await page.route(re.compile("^https://fonts.bbot/(.+)$"), fill_font)
            await page.goto(template_path)
            await page.set_content(html, wait_until="networkidle")
            await page.wait_for_timeout(5)
            img_raw = await page.locator(".video").screenshot(
                type="png",
            )
        return img_raw



import asyncio
import contextlib
import re
import time
from pathlib import Path
import json

import httpx
from graia.ariadne import Ariadne
from graiax.playwright.interface import PlaywrightContext
from loguru import logger
from playwright._impl._api_structures import Position
from playwright._impl._api_types import TimeoutError
from playwright.async_api._generated import BrowserContext, Page, Request, Response, Route
from sentry_sdk import capture_exception
from yarl import URL

from ..core.bot_config import BotConfig
from ..model.captcha import CaptchaResponse
from .fonts_provider import get_font

browser_cookies_file = Path("data").joinpath("browser_cookies.json")
error_path = Path("data").joinpath("error")
error_path.mkdir(parents=True, exist_ok=True)
captcha_path = Path("data").joinpath("captcha")
captcha_path.mkdir(parents=True, exist_ok=True)
mobile_style_js = Path(__file__).parent.parent.joinpath("static", "mobile_style.js")
font_mime_map = {
    "collection": "font/collection",
    "otf": "font/otf",
    "sfnt": "font/sfnt",
    "ttf": "font/ttf",
    "woff": "font/woff",
    "woff2": "font/woff2",
}


async def fill_font(route: Route, request: Request):
    url = URL(request.url)
    if not url.is_absolute():
        raise ValueError("字体地址不合法")
    try:
        logger.debug(f"Font {url.query['name']} requested")
        await route.fulfill(
            path=await get_font(url.query["name"]),
            content_type=font_mime_map.get(url.suffix),
        )
        return
    except Exception:
        logger.error(f"找不到字体 {url.query['name']}")
        await route.fallback()


async def resolve_select_captcha(page: Page):
    pass


async def browser_dynamic(dynid: str):
    app = Ariadne.current()
    browser_context = app.launch_manager.get_interface(PlaywrightContext).context
    # add cookies
    if browser_cookies_file.exists() and browser_cookies_file.is_file():
        if browser_cookies := json.loads(browser_cookies_file.read_bytes()):
            logger.debug(f"正在为浏览器添加cookies")
            await browser_context.add_cookies(
                [
                    {
                        "domain": cookie["domain"],
                        "name": cookie["name"],
                        "path": cookie["path"],
                        "value": cookie["value"],
                    }
                    for cookie in browser_cookies
                ]
            )
    return await screenshot(dynid, browser_context)


async def refresh_cookies(browser_context: BrowserContext):
    storage_state = await browser_context.storage_state()
    if cookies := storage_state.get("cookies"):
        browser_cookies_file.write_text(json.dumps(cookies))


async def screenshot(dynid: str, browser_context: BrowserContext, log=True):
    logger.info(f"正在截图动态：{dynid}")
    st = int(time.time())
    for i in range(3):
        page = await browser_context.new_page()
        await page.route(re.compile("^https://fonts.bbot/(.+)$"), fill_font)
        try:
            if log:
                page.on("requestfinished", network_request)
            page.on("requestfailed", network_requestfailed)
            if BotConfig.Bilibili.mobile_style:
                page, clip = await get_mobile_screenshot(page, dynid)
            else:
                page, clip = await get_pc_screenshot(page, dynid)
            clip["height"] = min(clip["height"], 32766)  # 限制高度
            if picture := await page.screenshot(
                clip=clip, full_page=True, type="jpeg", quality=98
            ):
                await refresh_cookies(browser_context)
                return picture
        except TimeoutError:
            logger.error(f"[BiliBili推送] {dynid} 动态截图超时，正在重试：")
        except Notfound:
            logger.error(f"[Bilibili推送] {dynid} 动态不存在，等待 3 秒后重试...")
            await asyncio.sleep(3)
        except AssertionError:
            logger.exception(f"[BiliBili推送] {dynid} 动态截图失败，正在重试：")
            await page.screenshot(
                path=f"{error_path}/{dynid}_{i}_{st}.jpg",
                full_page=True,
                type="jpeg",
                quality=80,
            )
        except Exception as e:  # noqa
            if "bilibili.com/404" in page.url:
                logger.error(f"[Bilibili推送] {dynid} 动态不存在")
                await refresh_cookies(browser_context)
                break
            elif "waiting until" in str(e):
                logger.error(f"[BiliBili推送] {dynid} 动态截图超时，正在重试：")
            else:
                capture_exception()
                logger.exception(f"[BiliBili推送] {dynid} 动态截图失败，正在重试：")
                try:
                    await page.screenshot(
                        path=f"{error_path}/{dynid}_{i}_{st}.jpg",
                        full_page=True,
                        type="jpeg",
                        quality=80,
                    )
                except Exception:
                    logger.exception(f"[BiliBili推送] {dynid} 动态截图失败：")
        finally:
            with contextlib.suppress():
                await page.close()


async def network_request(request: Request):
    url = request.url
    method = request.method
    response = await request.response()
    if response:
        status = response.status
        timing = "%.2f" % response.request.timing["responseEnd"]
    else:
        status = "/"
        timing = "/"
    logger.debug(f"[Response] [{method} {status}] {timing}ms <<  {url}")


def network_requestfailed(request: Request):
    url = request.url
    fail = request.failure
    method = request.method
    logger.warning(f"[RequestFailed] [{method} {fail}] << {url}")


async def get_mobile_screenshot(page: Page, dynid: str):
    url = f"https://m.bilibili.com/dynamic/{dynid}"
    captcha_image_body = ""
    last_captcha_id = ""
    captcha_result = None

    async def captcha_image_url_callback(response: Response):
        nonlocal captcha_image_body
        logger.debug(f"[Captcha] Get captcha image url: {response.url}")
        captcha_image_body = await response.body()

    async def captcha_result_callback(response: Response):
        nonlocal captcha_result, last_captcha_id
        logger.debug(f"[Captcha] Get captcha result: {response.url}")
        captcha_resp = await response.text()
        logger.debug(f"[Captcha] Result: {captcha_resp}")
        if '"result": "success"' in captcha_resp:
            logger.success("[Captcha] 验证码 Callback 验证成功")
            captcha_result = True
        elif '"result": "click"' in captcha_resp:
            pass
        else:
            if last_captcha_id:
                logger.warning(f"[Captcha] 验证码 Callback 验证失败，正在上报：{last_captcha_id}")
                async with httpx.AsyncClient() as client:
                    await client.post(
                        f"{captcha_baseurl}/report", json={"captcha_id": last_captcha_id}
                    )
                last_captcha_id = ""
            captcha_result = False

    await page.set_viewport_size({"width": 460, "height": 720})

    captcha_address = BotConfig.Bilibili.captcha_address
    if captcha_address:
        page.on(
            "response",
            lambda response: captcha_image_url_callback(response)
            if response.url.startswith("https://static.geetest.com/captcha_v3/")
            else None,
        )
        page.on(
            "response",
            lambda response: captcha_result_callback(response)
            if response.url.startswith("https://api.geetest.com/ajax.php")
            else None,
        )

    with contextlib.suppress(TimeoutError):
        await page.goto(url, wait_until="networkidle", timeout=20000)

    if captcha_address:
        captcha_baseurl = f"{captcha_address}/captcha/select"
        while captcha_image_body or captcha_result is False:
            logger.warning("[Captcha] 需要人机验证，正在尝试自动解决验证码")
            captcha_image = await page.query_selector(".geetest_item_img")
            assert captcha_image
            captcha_size = await captcha_image.bounding_box()
            assert captcha_size
            origin_image_size = 344, 384

            async with httpx.AsyncClient() as client:
                captcha_req = await client.post(
                    f"{captcha_baseurl}/bytes",
                    timeout=10,
                    files={"img_file": captcha_image_body},
                )
                captcha_req = CaptchaResponse(**captcha_req.json())
                logger.debug(f"[Captcha] Get Resolve Result: {captcha_req}")
                assert captcha_req.data
                last_captcha_id = captcha_req.data.captcha_id
            if captcha_req.data:
                click_points: list[list[int]] = captcha_req.data.points
                logger.warning(f"[Captcha] 识别到 {len(click_points)} 个坐标，正在点击")
                # 根据原图大小和截图大小计算缩放比例，然后计算出正确的需要点击的位置
                for point in click_points:
                    real_click_points = {
                        "x": point[0] * captcha_size["width"] / origin_image_size[0],
                        "y": point[1] * captcha_size["height"] / origin_image_size[1],
                    }
                    await captcha_image.click(position=Position(**real_click_points))
                    await page.wait_for_timeout(800)
                await page.click("text=确认")
                geetest_up = await page.wait_for_selector(".geetest_up", state="visible")
                await page.screenshot(path=captcha_path.joinpath(f"{last_captcha_id}.jpg"))
                if not geetest_up:
                    logger.warning("[Captcha] 未检测到验证码验证结果，正在重试")
                    continue
                geetest_result = await geetest_up.text_content()
                assert geetest_result
                logger.debug(f"[Captcha] Geetest result: {geetest_result}")
                if "验证成功" in geetest_result:
                    logger.success("[Captcha] 极验网页 Tip 验证成功")
                    captcha_image_body = ""
                else:
                    logger.warning("[Captcha] 极验验证失败，正在重试")

                with contextlib.suppress(TimeoutError):
                    await page.wait_for_load_state(state="domcontentloaded", timeout=20000)

    if "bilibili.com/404" in page.url:
        logger.warning(f"[Bilibili推送] {dynid} 动态不存在")
        raise Notfound

    await page.wait_for_load_state(state="domcontentloaded", timeout=20000)
    if "opus" in page.url:
        await page.wait_for_selector(".opus-module-author", state="visible")
    else:
        await page.wait_for_selector(".dyn-header__author__face", state="visible")

    await page.add_script_tag(path=mobile_style_js)
    await page.wait_for_function("getMobileStyle()")

    await page.evaluate(
        f"setFont('{BotConfig.Bilibili.dynamic_font}', '{BotConfig.Bilibili.dynamic_font_source}')"
    )

    # 判断字体是否加载完成
    await page.wait_for_timeout(
        200 if BotConfig.Bilibili.dynamic_font_source == "remote" else 50
    )
    need_wait = ["imageComplete", "fontsLoaded"]
    await asyncio.gather(*[page.wait_for_function(f"{i}()") for i in need_wait])

    card = await page.query_selector(".opus-modules" if "opus" in page.url else ".dyn-card")
    assert card
    clip = await card.bounding_box()
    assert clip
    logger.debug(f"loaded: {clip}")
    return page, clip


async def get_pc_screenshot(page: Page, dynid: str):
    url = f"https://t.bilibili.com/{dynid}"

    await page.set_viewport_size({"width": 2560, "height": 1080})
    with contextlib.suppress(TimeoutError):
        await page.goto(url, wait_until="networkidle", timeout=20000)

    if "bilibili.com/404" in page.url:
        logger.warning(f"[Bilibili推送] {dynid} 动态不存在")
        raise Notfound

    card = await page.query_selector(".card")
    assert card
    clip = await card.bounding_box()
    assert clip
    bar = await page.query_selector(".bili-dyn-action__icon")
    assert bar
    bar_bound = await bar.bounding_box()
    assert bar_bound
    clip["height"] = bar_bound["y"] - clip["y"] - 2
    logger.debug(f"loaded: {clip}")
    return page, clip


class Notfound(Exception):
    pass



from datetime import timedelta
from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.model import Member, Group
from graia.ariadne.message.element import At, Source
from graia.ariadne.message.chain import MessageChain
from graia.ariadne.event.message import GroupMessage
from graia.amnesia.builtins.memcache import Memcache
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.message.parser.twilight import (
    Twilight,
    FullMatch,
    RegexResult,
    WildcardMatch,
)

from ...core.control import Interval, Permission

channel = Channel.current()


@channel.use(
    ListenerSchema(
        listening_events=[GroupMessage],
        inline_dispatchers=[Twilight([FullMatch("/auth"), "anything" @ WildcardMatch()])],
        decorators=[Permission.require(Permission.GROUP_ADMIN), Interval.require()],
    )
)
async def main(
    app: Ariadne, group: Group, member: Member, source: Source, anything: RegexResult
):

    if anything.matched and (key := anything.result):

        memcache = app.launch_manager.get_interface(Memcache)
        if await memcache.has(key.display):
            if not await memcache.get(key.display):
                await memcache.set(key.display, str(member.id), timedelta(seconds=120))
                await app.send_group_message(
                    group, MessageChain(At(member), " 认证成功"), quote=source
                )
        else:
            await app.send_group_message(group, MessageChain(At(member), " 认证失败"), quote=source)



from loguru import logger
from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.exception import UnknownTarget
from graia.ariadne.message.chain import MessageChain
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.event.mirai import BotGroupPermissionChangeEvent

from ...core.bot_config import BotConfig

channel = Channel.current()


@channel.use(ListenerSchema(listening_events=[BotGroupPermissionChangeEvent]))
async def main(app: Ariadne, event: BotGroupPermissionChangeEvent):
    """
    群内权限变动
    """
    if BotConfig.Event.permchange:
        for admin in BotConfig.admins:
            try:
                await app.send_friend_message(
                    admin,
                    MessageChain(
                        "收到权限变动事件",
                        f"\n群号：{event.group.id}",
                        f"\n群名：{event.group.name}",
                        f"\n权限变更为：{event.current}",
                    ),
                )
            except UnknownTarget:
                logger.warning(f"由于未添加 {admin} 为好友，无法发送通知")



import aunly_bbot.function.command.add_talk
import aunly_bbot.function.command.announcement
import aunly_bbot.function.command.init
import aunly_bbot.function.command.quit_group
import aunly_bbot.function.command.status
import aunly_bbot.function.command.content_resolve
import aunly_bbot.function.command.vive_dynamic
import aunly_bbot.function.command.admin.add
import aunly_bbot.function.command.admin.remove
import aunly_bbot.function.command.configure.atall
import aunly_bbot.function.command.configure.nick
import aunly_bbot.function.command.menu
import aunly_bbot.function.command.subgroup.add_up
import aunly_bbot.function.command.subgroup.add
import aunly_bbot.function.command.subgroup.get_subgroup
import aunly_bbot.function.command.subgroup.remove_up
import aunly_bbot.function.command.subgroup.remove
import aunly_bbot.function.command.up.get_subscribe
import aunly_bbot.function.command.up.subscribe
import aunly_bbot.function.command.up.unsubscribe
import aunly_bbot.function.command.vip.add
import aunly_bbot.function.command.vip.remove
import aunly_bbot.function.command.whitelist.add
import aunly_bbot.function.command.whitelist.close
import aunly_bbot.function.command.whitelist.open
import aunly_bbot.function.command.whitelist.remove
import aunly_bbot.function.command.web_auth
import aunly_bbot.function.event.bot_launch
import aunly_bbot.function.event.exception
import aunly_bbot.function.event.invited_join_group
import aunly_bbot.function.event.join_group
import aunly_bbot.function.event.leave_group
import aunly_bbot.function.event.mute
import aunly_bbot.function.event.offline
import aunly_bbot.function.event.new_friend
import aunly_bbot.function.event.prem_change
import aunly_bbot.function.pusher.init
import aunly_bbot.function.pusher.dynamic
import aunly_bbot.function.pusher.live
import aunly_bbot.function.scheduler.version_update
import aunly_bbot.function.scheduler.refresh_token  # noqa

from loguru import logger

logger.success("[function] 加载完成")



from fastapi import APIRouter

from . import user, ws, auth, follow, home, config

router = APIRouter()

router.include_router(user.router, prefix="/user")
router.include_router(ws.router, prefix="/ws")
router.include_router(auth.router, prefix="/auth")
router.include_router(follow.router, prefix="/follow")
router.include_router(home.router, prefix="/home")
router.include_router(config.router, prefix="/config")



from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.model import Group
from graia.ariadne.message.element import At
from graia.ariadne.event.message import GroupMessage
from graia.ariadne.message.chain import MessageChain
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.message.parser.twilight import (
    Twilight,
    RegexMatch,
    RegexResult,
    ElementMatch,
    ElementResult,
    WildcardMatch,
)

from ....core import BOT_Status, Status
from ....core.bot_config import BotConfig
from ....utils.uid_extract import uid_extract
from ....utils.up_operation import subscribe_uid
from ....core.control import Interval, Permission

channel = Channel.current()


@channel.use(
    ListenerSchema(
        listening_events=[GroupMessage],
        inline_dispatchers=[
            Twilight(
                [
                    "at" @ ElementMatch(At),
                    RegexMatch(r"(订阅|关注)(主播|[uU][pP])?"),
                    "anything" @ WildcardMatch(optional=True),
                ]
            )
        ],
        decorators=[Permission.require(Permission.GROUP_ADMIN), Interval.require()],
    )
)
async def main(app: Ariadne, group: Group, at: ElementResult, anything: RegexResult):
    at_result: At = at.result  # type: ignore
    if at_result.target == app.account:
        if not BOT_Status.check_status(Status.INITIALIZED):
            return await app.send_group_message(
                group,
                MessageChain("正在初始化，请稍后..."),
            )
        message = anything.result.display  # type: ignore
        uid = await uid_extract(message, show_error=True)
        if uid and uid.isdigit():
            if not BOT_Status.check_status(Status.DYNAMIC_IDLE):
                await app.send_group_message(
                    group,
                    MessageChain("正在订阅，请稍后..."),
                )
            msg = await subscribe_uid(uid, group.id)
            await app.send_group_message(
                group,
                MessageChain(msg),
            )
            if BotConfig.Event.subscribe:
                await app.send_friend_message(
                    BotConfig.master,
                    MessageChain(f"群：{group.name}（{group.id}）\n正在订阅 UP：{uid}\n{msg}"),
                )
        else:
            await app.send_group_message(
                group,
                MessageChain(uid or "未找到该 UP，请输入正确的 UP 群内昵称、UP 名、UP UID或 UP 首页链接"),
            )



import json
import asyncio

from loguru import logger
from graia.saya import Channel
from grpc.aio import AioRpcError
from graia.ariadne.app import Ariadne
from sentry_sdk import capture_exception
from bilireq.exceptions import GrpcError
from graia.ariadne.model import Group, Member
from httpx._exceptions import TimeoutException
from graia.broadcast.exceptions import ExecutionStop
from graia.ariadne.event.message import GroupMessage
from graia.ariadne.message.chain import MessageChain
from graia.ariadne.message.element import Image, Source
from graia.saya.builtins.broadcast.schema import ListenerSchema

from ...core.bot_config import BotConfig
from ...model.exception import AbortError
from ...utils.column_resolve import get_cv
from ...core.data import ContentResolveData
from ...core.control import Interval, Permission
from ...utils.video_subtitle import get_subtitle
from ...utils.bilibili_parse import extract_bilibili_info
from ...utils.draw_bili_image import BiliVideoImage
from ...utils.text2image import rich_text2image, browser_text2image
from ...utils.bilibili_request import get_b23_url, grpc_get_view_info
from ...utils.content_summarise import column_summarise, subtitle_summarise

channel = Channel.current()


@channel.use(ListenerSchema(listening_events=[GroupMessage], decorators=[Permission.require()]))
async def main(
    app: Ariadne, group: Group, member: Member, message: MessageChain, source: Source
):
    bili_number = await extract_bilibili_info(message)
    if not bili_number:
        return

    if bili_number[:2] in ["BV", "bv", "av"]:
        try:
            aid, cid, bvid, title, video_info = await extract_video_info(bili_number)
        except AbortError as e:
            await Interval.manual(group.id, 5)
            return await app.send_group_message(group, MessageChain(e.message), quote=source)

        archive_data = ContentResolveData(aid=aid)
        archive_data.title = title
        await Interval.manual(aid + group.id, 30)
        try:
            logger.info(f"开始生成视频信息图片：{aid}")
            b23_url = await get_b23_url(f"https://www.bilibili.com/video/{bvid}")

            image = await (await BiliVideoImage.from_view_rely(video_info, b23_url)).render(BotConfig.Bilibili.render_style)
            info_message = await app.send_group_message(
                group,
                MessageChain(
                    Image(data_bytes=image),
                    f"\n{b23_url}",
                ),
                quote=source,
            )

            if info_message.id < 0:
                logger.warning(f"发送视频信息图片失败，可能是 Bot 被风控：{aid}")
                return

            if BotConfig.Bilibili.openai_summarization or BotConfig.Bilibili.use_wordcloud:
                try:
                    if archive_data.content:
                        subtitle = [str(x) for x in json.loads(archive_data.content)]
                    else:
                        subtitle = await get_subtitle(aid, cid)
                        archive_data.content = json.dumps(subtitle, ensure_ascii=False)

                    if (
                        len(subtitle) < 10
                        or video_info.arc.duration < BotConfig.Bilibili.asr_length_threshold
                    ):
                        raise AbortError("字幕内容过少且视频时长过短，跳过总结请求")

                    chatgpt_thinks = True

                    async def openai_summarization():
                        logger.info(f"开始进行 AI 总结：{aid}")
                        try:
                            if archive_data.openai:
                                logger.info(f"{aid} 已存在总结缓存，跳过总结请求")
                                summarise = archive_data.openai
                            else:
                                logger.info(f"{aid} 总结不存在，正在尝试请求......")
                                try:
                                    if (
                                        BotConfig.Bilibili.openai_whitelist_users
                                        and member.id
                                        not in BotConfig.Bilibili.openai_whitelist_users
                                    ):
                                        await Interval.manual(
                                            member, BotConfig.Bilibili.openai_cooldown
                                        )
                                except ExecutionStop:
                                    msg = f"{member.id} 在 10 分钟内已经请求过总结，跳过本次请求"
                                    logger.info(msg)
                                    raise AbortError(msg)
                                ai_summary = await subtitle_summarise(subtitle, title)
                                if ai_summary.response:
                                    summarise = ai_summary.response
                                    archive_data.openai = summarise
                                else:
                                    logger.warning(f"视频 {aid} 总结失败：{ai_summary.raw}")
                                    return

                            if "no meaning" in summarise.lower() or len(summarise) < 20:
                                nonlocal chatgpt_thinks
                                chatgpt_thinks = False
                                raise AbortError("ChatGPT 认为这些字幕没有意义")

                            logger.debug(summarise)
                            if BotConfig.Bilibili.use_browser:
                                image = await browser_text2image(summarise)
                            else:
                                image = await rich_text2image(summarise)
                            if image:
                                images.append(image)
                        except AbortError as e:
                            logger.warning(f"视频 {aid} 总结被终止：{e}")
                        except Exception:
                            capture_exception()
                            logger.exception(f"视频 {aid} 总结出错")

                    async def wordcloud():
                        logger.info(f"开始生成词云：{aid}")
                        try:
                            from ...utils.wordcloud import get_worldcloud_image, get_frequencies

                            if archive_data.jieba:
                                logger.info(f"{aid} 已存在分词缓存，跳过分词")
                                word_frequencies = dict(json.loads(archive_data.jieba))
                            else:
                                logger.info(f"{aid} 分词不存在，正在尝试分词......")
                                word_frequencies = await asyncio.to_thread(
                                    get_frequencies, subtitle
                                )
                                archive_data.jieba = json.dumps(
                                    word_frequencies, ensure_ascii=False
                                )

                            wordcloud = await get_worldcloud_image(word_frequencies)
                            if wordcloud:
                                images.append(wordcloud)
                        except Exception:
                            capture_exception()
                            logger.exception(f"视频 {aid} 词云出错")

                    images = []
                    if BotConfig.Bilibili.openai_summarization:
                        await openai_summarization()
                    if BotConfig.Bilibili.use_wordcloud and chatgpt_thinks:
                        await wordcloud()

                    if images:
                        await app.send_group_message(
                            group,
                            MessageChain([Image(data_bytes=x) for x in images]),
                            quote=info_message.source,
                        )

                except AbortError as e:
                    logger.warning(f"视频 {aid} 总结失败：{e.message}")
                    return
                archive_data.save()

        except TimeoutException:
            logger.exception(f"视频 {aid} 信息生成超时")
            await app.send_group_message(
                group, MessageChain(f"{bili_number} 视频信息生成超时，请稍后再试。"), quote=source
            )
        except Exception as e:
            capture_exception()
            logger.exception("视频解析 API 调用出错")
            await app.send_group_message(
                group, MessageChain(f"视频解析 API 调用出错：{e}"), quote=source
            )

    elif bili_number[:2] == "cv":
        if BotConfig.Bilibili.openai_summarization or BotConfig.Bilibili.use_wordcloud:
            column_id = bili_number[2:]
            archive_data = ContentResolveData(cid=column_id)
            try:
                if archive_data.content and archive_data.title:
                    cv_text = archive_data.content
                    cv_title = archive_data.title
                else:
                    cv_title, cv_text = await get_cv(column_id)
                    archive_data.title = cv_title
                    archive_data.content = cv_text

                async def openai_summarization():
                    try:
                        if archive_data.openai:
                            summarise = archive_data.openai
                        else:
                            ai_summary = await column_summarise(cv_title, cv_text)
                            if ai_summary.response:
                                summarise = ai_summary.response
                                archive_data.openai = summarise
                            else:
                                return

                        if BotConfig.Bilibili.use_browser:
                            image = await browser_text2image(summarise)
                        else:
                            image = await rich_text2image(summarise)
                        if image:
                            await app.send_group_message(
                                group, MessageChain(Image(data_bytes=image)), quote=source
                            )
                    except AbortError as e:
                        logger.warning(f"专栏 {column_id} 总结被终止：{e}")
                    except Exception:
                        capture_exception()
                        logger.exception(f"专栏 {column_id} 总结出错")

                async def wordcloud():
                    try:
                        from ...utils.wordcloud import get_worldcloud_image, get_frequencies

                        if archive_data.jieba:
                            word_frequencies = dict(json.loads(archive_data.jieba))
                        else:
                            word_frequencies = await asyncio.to_thread(
                                get_frequencies, [cv_title, cv_text]
                            )
                            archive_data.jieba = json.dumps(
                                word_frequencies, ensure_ascii=False
                            )

                        wordcloud = await get_worldcloud_image(word_frequencies)
                        if wordcloud:
                            await app.send_group_message(
                                group, MessageChain(Image(data_bytes=wordcloud)), quote=source
                            )
                    except Exception:
                        capture_exception()
                        logger.exception(f"专栏 {column_id} 词云出错")

                gather = []
                if BotConfig.Bilibili.openai_summarization:
                    gather.append(openai_summarization())
                if BotConfig.Bilibili.use_wordcloud:
                    gather.append(wordcloud())
                await asyncio.gather(*gather, return_exceptions=True)
            except AbortError as e:
                logger.warning(f"专栏 {column_id} 加载失败：{e.message}")
            archive_data.save()


def bv2av(bv):
    table = "fZodR9XQDSUm21yCkr6zBqiveYah8bt4xsWpHnJE7jL5VG3guMTKNPAwcF"
    tr = {table[i]: i for i in range(58)}
    s = [11, 10, 3, 8, 4, 6]
    xor = 177451812
    add = 8728348608
    r = sum(tr[bv[s[i]]] * 58**i for i in range(6))
    return (r - add) ^ xor


async def video_info_get(vid_id: str):
    if vid_id[:2].lower() == "av":
        aid = int(vid_id[2:])
        return await grpc_get_view_info(aid=aid) if aid > 1 else None
    return await grpc_get_view_info(bvid=vid_id)


async def extract_video_info(bili_number: str):
    try:
        if (video_info := await video_info_get(bili_number)) is None:
            raise AbortError(f"无法获取视频 {bili_number}。")
        elif video_info.ecode == 1:
            raise AbortError(f"未找到视频 {bili_number}，可能已被 UP 主删除。")
    except (AioRpcError, GrpcError) as e:
        logger.exception(e)
        raise AbortError(f"{bili_number} 视频信息获取失败，错误信息：{type(e)} {e}")
    except Exception as e:
        capture_exception()
        logger.exception(e)
        raise AbortError(f"{bili_number} 视频信息解析失败，错误信息：{type(e)} {e}")

    aid = video_info.activity_season.arc.aid or video_info.arc.aid
    cid = (
        video_info.activity_season.pages[0].page.cid
        if video_info.activity_season.pages
        else video_info.pages[0].page.cid
    )
    bvid = video_info.activity_season.bvid or video_info.bvid
    title = video_info.activity_season.arc.title or video_info.arc.title

    return aid, cid, bvid, title, video_info



from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.model import Friend
from graia.ariadne.message.chain import MessageChain
from graia.ariadne.event.message import FriendMessage
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.message.parser.twilight import (
    Twilight,
    FullMatch,
    RegexResult,
    WildcardMatch,
)

from ....core.control import Permission
from ....core.bot_config import BotConfig

channel = Channel.current()


@channel.use(
    ListenerSchema(
        listening_events=[FriendMessage],
        inline_dispatchers=[
            Twilight(
                [FullMatch("移除管理员"), "adminid" @ WildcardMatch(optional=True)],
            )
        ],
    )
)
async def main(app: Ariadne, friend: Friend, adminid: RegexResult):
    Permission.manual(friend, Permission.MASTER)
    if adminid.matched:
        say = adminid.result.display
        if say.isdigit():
            if int(say) in BotConfig.admins:
                BotConfig.admins.remove(int(say))
                BotConfig.save()
                await app.send_friend_message(friend, MessageChain("成功取消该账号的管理员"))
            else:
                await app.send_friend_message(friend, MessageChain("该账号不是管理员"))
        else:
            await app.send_friend_message(friend, MessageChain("管理员账号仅可为数字"))
    else:
        await app.send_friend_message(friend, MessageChain("未输入管理员账号"))



from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.model import Friend
from graia.ariadne.message.chain import MessageChain
from graia.ariadne.event.message import FriendMessage
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.message.parser.twilight import (
    Twilight,
    FullMatch,
    RegexResult,
    WildcardMatch,
)

from ....core.control import Permission
from ....core.subgroup_config import SubGroup

channel = Channel.current()


@channel.use(
    ListenerSchema(
        listening_events=[FriendMessage],
        inline_dispatchers=[
            Twilight(
                [FullMatch("移除订阅组"), "group_name" @ WildcardMatch(optional=True)],
            )
        ],
    )
)
async def main(app: Ariadne, friend: Friend, group_name: RegexResult):
    Permission.manual(friend, Permission.MASTER)
    if group_name.matched:
        say = group_name.result.display
        sg = SubGroup(say)
        if sg.remove_from_group_names():
            await app.send_friend_message(friend, MessageChain(f"成功将该名称 [{say}] 移除订阅组"))
        else:
            await app.send_friend_message(friend, MessageChain(f"该名称 [{say}] 并不在订阅组中"))
    else:
        await app.send_friend_message(friend, MessageChain("未输入订阅组名称"))



import random

from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.model import Friend
from graia.ariadne.exception import RemoteException
from graia.ariadne.message.chain import MessageChain
from graia.ariadne.event.message import FriendMessage
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.message.parser.twilight import Twilight, FullMatch

from ...core import BOT_Status, Status
from ...core.control import Permission

channel = Channel.current()


@channel.use(
    ListenerSchema(
        listening_events=[FriendMessage],
        inline_dispatchers=[
            Twilight(
                [FullMatch("/init")],
            )
        ],
    )
)
async def main(app: Ariadne, friend: Friend):
    Permission.manual(friend, Permission.MASTER)
    if BOT_Status.check_status(Status.INITIALIZED):
        await app.send_friend_message(friend, MessageChain("已初始化，无需重复操作"))
    else:
        group = random.choice(await app.get_group_list())
        try:
            await app.recall_message(await app.send_group_message(group, MessageChain("test")))
            BOT_Status.set_status(Status.INITIALIZED, True)
            await app.send_friend_message(friend, MessageChain("初始化成功"))
        except RemoteException as e:
            await app.send_friend_message(friend, MessageChain(f"初始化失败，{e}"))



import time
import asyncio

from loguru import logger
from graia.saya import Channel
from json import JSONDecodeError
from httpx import TransportError
from graia.ariadne.app import Ariadne
from sentry_sdk import capture_exception
from graia.ariadne.model import MemberPerm
from bilireq.live import get_rooms_info_by_uids
from graia.broadcast.exceptions import ExecutionStop
from graia.ariadne.message.chain import MessageChain
from graia.ariadne.connection.util import UploadMethod
from graia.ariadne.message.element import Image, AtAll
from graia.scheduler.saya.schema import SchedulerSchema
from graia.scheduler.timers import every_custom_seconds
from graia.ariadne.exception import UnknownTarget, AccountMuted, RemoteException

from ...core import BOT_Status, Status
from ...core.bot_config import BotConfig
from ...utils.time_tools import calc_time_total
from ...utils.bilibili_request import get_b23_url
from ...utils.up_operation import delete_group, delete_uid, set_name
from ...core.data import insert_live_push, get_all_uid, get_sub_by_uid, update_cover_by_id

channel = Channel.current()


@channel.use(SchedulerSchema(every_custom_seconds(BotConfig.Bilibili.dynamic_interval)))
async def main(app: Ariadne):
    if (
        not BOT_Status.is_all_statuses_true(
            Status.INITIALIZED, Status.ACCOUNT_CONNECTED, Status.STARTED, Status.SUBSCRIBE_IDLE
        )
        or BOT_Status.is_all_statuses_true(
            Status.INITIALIZED, Status.ACCOUNT_CONNECTED, Status.STARTED, Status.SUBSCRIBE_IDLE
        )
        and len(get_all_uid()) == 0
    ):
        await asyncio.sleep(5)
        return
    sub_list = get_all_uid()

    BOT_Status.set_status(Status.LIVE_IDLE, False)

    for up in BOT_Status.living.copy():
        if up not in sub_list:
            del BOT_Status.living[up]
    try:
        try:
            status_infos = await get_rooms_info_by_uids(sub_list)
        except (TransportError, JSONDecodeError) as error:
            logger.warning(f"获取直播间状态失败: {type(error)} {error}")
            await asyncio.sleep(5)
            BOT_Status.set_status(Status.LIVE_IDLE, True)
            return
        except RuntimeError as error:
            if "The connection pool was closed while" not in str(error):
                raise error
            BOT_Status.set_status(Status.LIVE_IDLE, True)
            return
    except Exception:  # noqa
        capture_exception()
        logger.exception("获取直播间状态失败:")
        await asyncio.sleep(30)
        BOT_Status.set_status(Status.LIVE_IDLE, True)
        return
    if status_infos:
        for up, live_data in status_infos.items():
            up_name = live_data["uname"]
            up_id = up
            # 检测订阅配置里是否有该 up
            if up_id in sub_list:
                # 如果存在直播信息则为已开播
                logger.debug(
                    f"[Live] {up_name}({up_id}) live_status: {live_data['live_status']}"
                )
                if live_data["live_status"] == 1:
                    # 判断是否在正在直播列表中
                    if up_id in BOT_Status.living:
                        continue
                    logger.debug(f"[Live] {up} live_data: {live_data}")
                    BOT_Status.living[up_id] = live_data["live_time"]  # 设定开播时间
                    # 获取直播信息
                    room_id = live_data["room_id"]
                    title = live_data["title"]
                    area_parent = live_data["area_v2_parent_name"]
                    area = live_data["area_v2_name"]
                    room_area = f"{area_parent} / {area}"
                    cover_img_url = update_cover_by_id(
                        up_id, live_data.get("cover_from_user", live_data.get("keyframe"))
                    )
                    cover_img = (
                        await app.upload_image(
                            await Image(url=cover_img_url).get_bytes(),
                            UploadMethod.Group,
                        )
                        if cover_img_url
                        else None
                    )
                    set_name(up_id, up_name)
                    logger.info(f"[BiliBili推送] {up_name} 开播了 - {room_area} - {title}")

                    # 发送推送消息
                    for data in get_sub_by_uid(up_id):
                        if (  # 判断是否处于 DEBUG 模式
                            BotConfig.Debug.enable
                            and int(data.group) not in BotConfig.Debug.groups
                        ):
                            continue
                        if data.live:  # 判断该群是否开启直播推送
                            group = await app.get_group(int(data.group))
                            nick = (
                                f"*{up_nick} "
                                if (up_nick := data.nick)
                                else f"UP {up_name}（{up_id}）"
                            )
                            msg = [
                                f"{nick}在 {room_area} 区开播啦 ！\n标题：{title}\n",
                                cover_img or "",
                                "\n" if cover_img else "",
                                await get_b23_url(f"https://live.bilibili.com/{room_id}"),
                            ]

                            if data.atall:  # 判断是否开启@全体推送
                                bot_perm = group.account_perm if group else MemberPerm.Member
                                if bot_perm in [
                                    MemberPerm.Administrator,
                                    MemberPerm.Owner,
                                ]:
                                    msg = [AtAll(), " "] + msg
                                else:
                                    msg = ["@全体成员 "] + msg
                            try:
                                await app.send_group_message(
                                    int(data.group),
                                    MessageChain(msg),
                                )
                            except UnknownTarget:
                                delete = await delete_group(data.group)
                                logger.info(
                                    f"[BiliBili推送] 推送失败，找不到该群 {data.group}，已删除该群订阅的 {len(delete)} 个 UP"
                                )
                            except AccountMuted:
                                group = f"{group.name}（{group.id}）" if group else data.group
                                logger.warning(f"[BiliBili推送] 推送失败，账号在 {group} 被禁言")
                            await asyncio.sleep(5)

                    insert_live_push(
                        up_id, True, len(get_sub_by_uid(up_id)), title, area_parent, area
                    )
                elif up_id in BOT_Status.living:
                    logger.debug(f"[Live] {up} live_data: {live_data}")
                    live_time = (
                        f"，本次直播时长 {calc_time_total(time.time() - BOT_Status.living[up_id])}。"
                        if BOT_Status.living[up_id]
                        else "。"
                    )
                    del BOT_Status.living[up_id]
                    set_name(up_id, up_name)
                    logger.info(f"[BiliBili推送] {up_name} 已下播{live_time}")
                    for data in get_sub_by_uid(up_id):
                        if (
                            BotConfig.Debug.enable
                            and int(data.group) not in BotConfig.Debug.groups
                        ):
                            continue
                        if data.live:
                            try:
                                nick = (
                                    f"*{up_nick} "
                                    if (up_nick := data.nick)
                                    else f"UP {up_name}（{up_id}）"
                                )
                                await app.send_group_message(
                                    int(data.group),
                                    MessageChain(f"{nick}已下播{live_time}"),
                                )
                            except UnknownTarget:
                                delete = await delete_group(data.group)
                                logger.info(
                                    f"[BiliBili推送] 推送失败，找不到该群 {data.group}，已删除该群订阅的 {len(delete)} 个 UP"
                                )
                            except AccountMuted:
                                group = await app.get_group(int(data.group))
                                group = f"{group.name}（{group.id}）" if group else data.group
                                logger.warning(f"[BiliBili推送] 推送失败，账号在 {group} 被禁言")
                            except RemoteException as e:
                                if "resultType=46" in str(e):
                                    logger.error("[BiliBili推送] 推送失败，Bot 被限制发送群聊消息")
                                    await app.send_friend_message(
                                        BotConfig.master,
                                        MessageChain(
                                            "Bot 被限制发送群聊消息（46 代码），请尽快处理后发送 /init 重新开启推送进程"
                                        ),
                                    )
                                    BOT_Status.set_status(Status.LIVE_IDLE, True)
                                    BOT_Status.set_status(Status.INITIALIZED, True)
                                    raise ExecutionStop() from e
                                else:
                                    capture_exception()
                                    logger.exception("[BiliBili推送] 推送失败，未知错误")
                            except Exception:  # noqa
                                capture_exception()
                                logger.exception("[BiliBili推送] 推送失败，未知错误")
                            await asyncio.sleep(5)
                    insert_live_push(up_id, False, len(get_sub_by_uid(up_id)))
            else:
                logger.warning(f"[BiliBili推送] 未找到订阅 UP {up_name}（{up_id}）的群，正在退订！")
                await delete_uid(up_id)

    BOT_Status.set_status(Status.LIVE_IDLE, True)



import traceback

from io import StringIO
from graia.saya import Channel
from graia.ariadne.app import Ariadne
from sentry_sdk import capture_exception
from graia.ariadne.message.element import Image
from graia.ariadne.message.chain import MessageChain
from graia.broadcast.builtin.event import ExceptionThrowed
from graia.saya.builtins.broadcast.schema import ListenerSchema

from ...core.bot_config import BotConfig
from ...utils.text2image import text2image


channel = Channel.current()


async def make_msg_for_unknown_exception(event: ExceptionThrowed):
    capture_exception(event.exception)
    with StringIO() as fp:
        traceback.print_tb(event.exception.__traceback__, file=fp)
        tb = fp.getvalue()
    msg = str(
        f"异常事件：\n{str(event.event)}"
        + f"\n \n异常类型：\n{type(event.exception)}"
        + f"\n \n异常内容：\n{str(event.exception)}"
        + f"\n \n异常追踪：\n{tb}\n{str(event.exception)}"
    )
    image = await text2image(msg, 200)
    return MessageChain("发生未捕获的异常\n", Image(data_bytes=image))


@channel.use(ListenerSchema(listening_events=[ExceptionThrowed]))
async def main(event: ExceptionThrowed):
    if isinstance(event.event, ExceptionThrowed):
        return
    app = Ariadne.current()
    eimg = await make_msg_for_unknown_exception(event)
    return await app.send_friend_message(
        BotConfig.master,
        eimg,
    )



import asyncio

from loguru import logger
from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.model import Group
from graia.ariadne.exception import UnknownTarget
from graia.ariadne.message.chain import MessageChain
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.event.mirai import (
    BotLeaveEventKick,
    BotLeaveEventActive,
    BotLeaveEventDisband,
)

from ...core.bot_config import BotConfig
from ...core.data import get_sub_by_group
from ...core.group_config import GroupPermission
from ...utils.up_operation import unsubscribe_uid

channel = Channel.current()


@channel.use(
    ListenerSchema(
        listening_events=[BotLeaveEventKick, BotLeaveEventActive, BotLeaveEventDisband]
    )
)
async def main(app: Ariadne, group: Group):
    GroupPermission(group).remove_from_whitelist()
    logger.info(f"[BiliBili推送] 检测到退群事件 > {group.name}({group.id})，正在删除该群的订阅")
    remove_list = []
    for data in get_sub_by_group(group.id):
        await unsubscribe_uid(data.uid, group.id)
        remove_list.append(data.uid)
        logger.info(f"[BiliBili推送] 已删除订阅 > {data.uid}")
        await asyncio.sleep(2)
    logger.info(
        f"[BiliBili推送] 检测到退群事件 > {group.name}({group.id})，已删除该群订阅的 {len(remove_list)} 个 UP"
    )
    for admin in BotConfig.admins:
        try:
            await app.send_friend_message(
                admin,
                MessageChain(
                    "收到退出群聊事件",
                    f"\n群号：{group.id}",
                    f"\n群名：{group.name}",
                    f"\n已移出白名单并删除该群订阅的 {len(remove_list)} 个 UP",
                ),
            )
        except UnknownTarget:
            logger.warning(f"由于未添加 {admin} 为好友，无法发送通知")



import click

from pydantic import ValidationError

from ..model.config import _BotConfig


def valueerror_output(err: dict):
    err_info = []
    pos_maxlen = 0
    for err_pos, err_msg in err.items():
        pos_maxlen = max(pos_maxlen, len(err_pos))
        err_info.append([err_pos, err_msg])
    click.secho("以下配置项填写错误: ", fg="bright_red")
    for err in err_info:
        click.secho(f"{err[0].ljust(pos_maxlen)} => {err[1]}", fg="bright_red")


while True:
    try:
        BotConfig: _BotConfig = _BotConfig.load()
        break
    except ValidationError as e:
        valueerror_output(_BotConfig.valueerror_parser(e))
    except FileNotFoundError:
        click.secho("未找到配置文件, 正在进入配置流程......", fg="bright_red", bold=True)
    except Exception as e:
        click.secho(f"未知错误: {e}", fg="bright_red", bold=True)

    from ..cli.config import click_config

    click_config()



import httpx
import random
import asyncio
import tiktoken_async

from loguru import logger
from httpx import Response
from typing import Optional
from collections import OrderedDict

from ..core.bot_config import BotConfig
from ..model.openai import OpenAI, TokenUsage

LIMIT_COUNT = {
    "gpt-3.5-turbo-0613": 3500,
    "gpt-3.5-turbo-16k-0613": 15000,
    "gpt-4-0613": 7600,
}.get(BotConfig.Bilibili.openai_model or "gpt-3.5-turbo-0613", 3500)

if BotConfig.Bilibili.openai_summarization:
    logger.info("正在加载 OpenAI Token 计算模型")
    tiktoken_enc = asyncio.run(
        tiktoken_async.encoding_for_model(BotConfig.Bilibili.openai_model)
    )
    logger.info(f"{tiktoken_enc.name} 加载成功")


def get_summarise_prompt(title: str, transcript: str) -> list[dict[str, str]]:
    title = title.replace("\n", " ").strip() if title else ""
    transcript = transcript.replace("\n", " ").strip() if transcript else ""
    if BotConfig.Bilibili.openai_promot_version == 1:
        language = "Chinese"
        sys_prompt = (
            "Your output should use the following template:\n## Summary\n## Highlights\n"
            "- [Emoji] Bulletpoint\n\n"
            "Your task is to summarise the video I have given you in up to 2 to 6 concise bullet points. "
            "First, use a simple sentence to summarize, each bullet point is at least 15 words. "
            "Choose an appropriate emoji for each bullet point. "
            "Use the video above: {{Title}} {{Transcript}}."
            "\nIf you think that the content in the transcript is meaningless, "
            "Or if there is very little content that cannot be well summarized, "
            "then you can simply output the two words 'no meaning'. Remember, not to output anything else."
        )
        return get_full_prompt(
            f'Title: "{title}"\nTranscript: "{transcript}"', sys_prompt, language
        )
    return get_full_prompt(
        prompt=(
            "使用以下Markdown模板为我总结视频字幕数据，除非字幕中的内容无意义，或者内容较少无法总结，或者未提供字幕数据，或者无有效内容，你就不使用模板回复，只回复“无意义”："
            "\n## 概述"
            "\n{内容，尽可能精简总结内容不要太详细}"
            "\n## 要点"
            "\n- {使用不重复并合适的emoji，仅限一个，禁止重复} {内容不换行大于15字，可多项，条数与有效内容数量呈正比}"
            "\n不要随意翻译任何内容。仅使用中文总结。"
            "\n不说与总结无关的其他内容，你的回复仅限固定格式提供的“概述”和“要点”两项。"
            f"\n视频标题名称为“{title}”，视频字幕数据如下，立刻开始总结：“{transcript}”"
        )
    )


def count_tokens(prompts: list[dict[str, str]]):
    """根据内容计算 token 数"""

    if BotConfig.Bilibili.openai_model.startswith("gpt-3.5-turbo"):
        tokens_per_message = 4
        tokens_per_name = -1
    elif BotConfig.Bilibili.openai_model.startswith("gpt-4"):
        tokens_per_message = 3
        tokens_per_name = 1
    else:
        raise ValueError(f"Unknown model name {BotConfig.Bilibili.openai_model}")

    num_tokens = 0
    for message in prompts:
        num_tokens += tokens_per_message
        for key, value in message.items():
            num_tokens += len(tiktoken_enc.encode(value))
            if key == "name":
                num_tokens += tokens_per_name
    num_tokens += 3
    return num_tokens


def get_small_size_transcripts(text_data: list[str], token_limit: int = LIMIT_COUNT):
    unique_texts = list(OrderedDict.fromkeys(text_data))
    while count_tokens(get_summarise_prompt("", " ".join(unique_texts))) > token_limit:
        unique_texts.pop(random.randint(0, len(unique_texts) - 1))
    return " ".join(unique_texts)


def get_full_prompt(
    prompt: Optional[str] = None, system: Optional[str] = None, language: Optional[str] = None
):
    plist: list[dict[str, str]] = []
    if system:
        plist.append({"role": "system", "content": system})
    if prompt:
        plist.append({"role": "user", "content": prompt})
    if language:
        plist.extend(
            (
                {
                    "role": "assistant",
                    "content": "What language do you want to output?",
                },
                {"role": "user", "content": language},
            )
        )
    if not plist:
        raise ValueError("No prompt provided")
    return plist


async def openai_req(
    prompt_message: list[dict[str, str]],
    token: Optional[str] = BotConfig.Bilibili.openai_api_token,
    model: str = BotConfig.Bilibili.openai_model,
    temperature: Optional[float] = None,
) -> OpenAI:
    if not token:
        return OpenAI(error=True, message="未配置 OpenAI API Token")
    async with httpx.AsyncClient(
        proxies=BotConfig.Bilibili.openai_proxy,
        headers={
            "Authorization": f"Bearer {token}",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko)"
            " Chrome/110.0.0.0 Safari/537.36 Edg/110.0.1587.69",
        },
        timeout=100,
    ) as client:
        data = {
            "model": model,
            "messages": prompt_message,
        }
        if temperature:
            data["temperature"] = temperature
        try:
            req: Response = await client.post(
                "https://api.openai.com/v1/chat/completions", json=data
            )
        except Exception as e:
            return OpenAI(error=True, message=f"OpenAI 请求失败 {type(e)} {e}")
        if req.status_code != 200:
            return OpenAI(error=True, message=req.text, raw=req.json())
        logger.info(f"[OpenAI] Response:\n{req.json()['choices'][0]['message']['content']}")
        usage = req.json()["usage"]
        logger.info(f"[OpenAI] Response 实际 token 消耗: {usage}")
        return OpenAI(
            response=req.json()["choices"][0]["message"]["content"],
            raw=req.json(),
            token_usage=TokenUsage(**usage),
        )



import httpx
import contextlib

from pathlib import Path
from importlib import metadata
from packaging.version import parse


def get_local_version():
    TOML_PATH = Path(__file__).parent.parent.parent.joinpath("pyproject.toml")

    RAW_TOML = TOML_PATH.read_text(encoding="utf-8") if TOML_PATH.exists() else ""

    return (
        RAW_TOML.split("version = ")[1].split("\n")[0].strip('"')
        if RAW_TOML
        else metadata.version("aunly_bbot")
    )


async def get_remote_version():
    with contextlib.suppress(Exception):
        async with httpx.AsyncClient() as client:
            resp = await client.get("https://pypi.org/pypi/aunly-bbot/json")
            resp = resp.json()
            # 按照时间排序，取最新的版本
            return sorted(resp["releases"].keys(), key=parse)


async def update_version():
    local_version = get_local_version()
    remote_version = await get_remote_version()

    if local_version and remote_version and parse(local_version) < parse(remote_version[-1]):
        return parse(local_version), parse(remote_version[-1])



from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.message.element import At
from graia.ariadne.model import Group, MemberPerm
from graia.ariadne.event.message import GroupMessage
from graia.ariadne.message.chain import MessageChain
from graia.saya.builtins.broadcast.schema import ListenerSchema
from graia.ariadne.message.parser.twilight import (
    Twilight,
    RegexMatch,
    ParamMatch,
    RegexResult,
    ElementMatch,
    ElementResult,
)

from ....core.bot_config import BotConfig
from ....utils.up_operation import set_atall
from ....utils.uid_extract import uid_extract
from ....core.control import Interval, Permission

channel = Channel.current()


@channel.use(
    ListenerSchema(
        listening_events=[GroupMessage],
        inline_dispatchers=[
            Twilight(
                [
                    "at" @ ElementMatch(At, optional=True),
                    "act" @ RegexMatch(r"开启|关闭"),
                    RegexMatch(r"@?全体(成员)?"),
                    "uid" @ ParamMatch(optional=True),
                ]
            )
        ],
        decorators=[Permission.require(Permission.GROUP_ADMIN), Interval.require()],
    )
)
async def main(
    app: Ariadne, group: Group, at: ElementResult, act: RegexResult, uid: RegexResult
):
    if at.result:
        at_element: At = at.result  # type: ignore
        if at_element.target != BotConfig.Mirai.account:
            return

    if uid.matched:
        uid = await uid_extract(uid.result.display, group.id)
        if uid:
            acts = act.result.display
            if acts == "开启":
                if group.account_perm in [MemberPerm.Administrator, MemberPerm.Owner]:
                    msg = f"{uid} @全体开启成功" if set_atall(uid, group.id, True) else "该群未关注此 UP"
                else:
                    msg = "Bot 权限不足，无法开启@全体，请赋予 Bot 管理员权限或更高"
            else:
                msg = f"{uid} @全体关闭成功" if set_atall(uid, group.id, False) else "该群未关注此 UP"
        else:
            msg = "请输入正确的 UID"
    else:
        msg = "请输入正确的 UID"

    await app.send_group_message(group, MessageChain(msg))



import os
import sys
import httpx

from creart import it
from pathlib import Path
from graia.saya import Saya
from fastapi import FastAPI
from graia.ariadne.app import Ariadne
from graia.scheduler import GraiaScheduler
from graia.amnesia.builtins.uvicorn import UvicornService
from graia.amnesia.builtins.memcache import MemcacheService
from graia.ariadne.entry import config, HttpClientConfig, WebsocketClientConfig

from .core import cache
from .core.log import logger
from .website import BotWebService
from .core.bot_config import BotConfig
from .utils.fastapi import FastAPIService
from .utils.verify_mah import verify_mirai
from .utils.fonts_provider import font_init
from .utils.detect_package import is_package
from .core.announcement import base_telemetry


os.environ["PLAYWRIGHT_BROWSERS_PATH"] = (
    "0" if is_package else Path(__file__).parent.joinpath("static", "browser").as_posix()
)
logger.info("BBot is starting...")

base_telemetry()

logger.info("正在下载字体...")
try:
    font_init()
except httpx.TimeoutException:
    logger.error("获取字体超时，请手动下载后放置于（data/font）目录或稍后重试")
    sys.exit()
logger.success("字体下载完成！")

host = BotConfig.Mirai.mirai_host
if cache.get("skip_verify"):
    if verify_mirai(host, BotConfig.Mirai.account, BotConfig.Mirai.verify_key):
        logger.success("Mirai HTTP API 验证成功！")
    else:
        sys.exit(1)
app_config = config(
    BotConfig.Mirai.account,
    BotConfig.Mirai.verify_key,
    HttpClientConfig(host),
    WebsocketClientConfig(host),
)

app = Ariadne(app_config)
app.config(install_log=True)

if BotConfig.Bilibili.use_browser:
    from graiax.playwright.service import PlaywrightService  # type: ignore # noqa

    app.launch_manager.add_service(
        PlaywrightService(
            browser_type="chromium",
            user_data_dir=Path("data").joinpath("browser"),
            device_scale_factor=1.5 if BotConfig.Bilibili.mobile_style else 1.25,
            user_agent=(
                "Mozilla/5.0 (Linux; Android 10; RMX1911) AppleWebKit/537.36 "
                "(KHTML, like Gecko) Chrome/100.0.4896.127 Mobile Safari/537.36"
            )
            if BotConfig.Bilibili.mobile_style
            else "",
        )
    )
app.launch_manager.add_service(MemcacheService())
app.launch_manager.add_service(
    FastAPIService(
        FastAPI(
            title="BBot API",
            description="适用于 BBot WebUI 的 API 文档",
            swagger_ui_parameters={"defaultModelsExpandDepth": -1},
        )
    )
)
app.launch_manager.add_service(
    UvicornService(host=BotConfig.Webui.webui_host, port=int(BotConfig.Webui.webui_port))
)
app.launch_manager.add_service(BotWebService())

app.create(GraiaScheduler)
saya = it(Saya)


with saya.module_context():
    if cache.get("test"):
        logger.info("正在进入测试模式...")
        saya.require("aunly_bbot.test")
    else:
        saya.require("aunly_bbot.function")
        from . import function  # noqa

        custom_path = Path("data", "custom")
        custom_path.mkdir(parents=True, exist_ok=True)
        for module in custom_path.glob("*"):
            if module.name != "__pycache__":
                logger.info(f"正在加载自定义模块：{module.stem}")
                saya.require(f"data.custom.{module.stem}")



import re
import contextlib

from pathlib import Path
from loguru import logger
from importlib import metadata

try:
    package = metadata.requires("aunly_bbot")
except Exception:
    package = None


TOML_PATH = Path(__file__).parent.parent.parent.joinpath("pyproject.toml")

RAW_TOML = TOML_PATH.read_text(encoding="utf-8") if TOML_PATH.exists() else ""

PROJECT_VERSION = (
    re.search(r'version = "([^"]+)"', RAW_TOML)[1]  # type: ignore
    if RAW_TOML
    else metadata.version("aunly_bbot")
)
ARIADNE_VERSION = (
    # (RAW_TOML.split("graia-ariadne[standard]")[1].split("=")[1].split('"')[0])
    re.search(r'graia-ariadne.+=(.+)"', RAW_TOML)[1]  # type: ignore
    if RAW_TOML
    else metadata.version("graia-ariadne")
)

BBOT_ASCII_LOGO = rf"""
                     //
         \\         //
          \\       //
    ##DDDDDDDDDDDDDDDDDDDDDD##
    ## DDDDDDDDDDDDDDDDDDDD ##   ________  ________  ________  _________
    ## hh                hh ##  |\   __  \|\   __  \|\   __  \|\___   ___\
    ## hh    //    \\    hh ##  \ \  \|\ /\ \  \|\ /\ \  \|\  \|___ \  \_|
    ## hh   //      \\   hh ##   \ \   __  \ \   __  \ \  \\\  \   \ \  \
    ## hh                hh ##    \ \  \|\  \ \  \|\  \ \  \\\  \   \ \  \
    ## hh      wwww      hh ##     \ \_______\ \_______\ \_______\   \ \__\
    ## hh                hh ##      \|_______|\|_______|\|_______|    \|__|
    ## MMMMMMMMMMMMMMMMMMMM ##
    ##MMMMMMMMMMMMMMMMMMMMMM##  Release {PROJECT_VERSION}. Powered by graia-ariadne {ARIADNE_VERSION}.
        \/            \/"""


def get_monitored_libs():
    libs = re.findall(
        r'"([\w\d-]+)(\[.*\])?<.+=(.+)"',
        re.findall(r"dependencies = \[\n(.*)\n\]", RAW_TOML, re.S)[0],
    )
    return {lib[0].lower(): lib[2] for lib in libs}


def get_dist_map() -> dict[str, str]:
    """获取与项目相关的发行字典"""

    dist_map: dict[str, str] = {}
    if package:
        for dist in package:
            dist = dist.split()[0].split(";")[0].split("=")[0].split("<")[0].split("[")[0]

            with contextlib.suppress(Exception):
                dist_map[dist] = metadata.version(dist)
    else:
        monitored_libs = get_monitored_libs()
        for dist in metadata.distributions():
            name: str = dist.metadata["Name"]
            if name.lower() in monitored_libs.keys():
                version: str = dist.version
                dist_map[name] = max(version, dist_map.get(name, ""))

    return dist_map


def base_telemetry() -> None:
    """执行基础遥测检查"""
    output: list[str] = [""]
    dist_map: dict[str, str] = get_dist_map()
    output.extend(
        " ".join(
            [
                f"[magenta]{name}[/]:",
                f"[green]{version}[/]",
            ]
        )
        for name, version in dist_map.items()
    )
    output.sort()
    output.insert(0, f"[cyan]{BBOT_ASCII_LOGO}[/]")
    rich_output = "\n".join(output)
    logger.opt(colors=True).info(
        rich_output.replace("[", "<").replace("]", ">"), alt=rich_output, highlighter=None
    )



from loguru import logger
from typing import Optional
from graia.saya import Channel
from graia.ariadne.app import Ariadne
from graia.ariadne.exception import UnknownTarget
from graia.ariadne.message.chain import MessageChain
from graia.ariadne.event.mirai import NewFriendRequestEvent
from graia.saya.builtins.broadcast.schema import ListenerSchema

from ...core.bot_config import BotConfig

channel = Channel.current()


@channel.use(ListenerSchema(listening_events=[NewFriendRequestEvent]))
async def main(app: Ariadne, event: NewFriendRequestEvent):
    """
    收到新好友事件
    """
    source_group: Optional[int] = event.source_group
    group_name = await app.get_group(source_group) if source_group else None
    group_name = group_name.name if group_name else "未知"
    for admin in BotConfig.admins:
        try:
            await app.send_friend_message(
                admin,
                MessageChain(
                    "收到添加好友事件",
                    f"\nQQ：{event.supplicant}",
                    f"\n昵称：{event.nickname}",
                    f"\n来自群：{group_name}({source_group})" if source_group else "\n来自好友搜索",
                    "\n状态：已通过申请\n",
                    event.message or "无附加信息",
                ),
            )
        except UnknownTarget:
            logger.warning(f"由于未添加 {admin} 为好友，无法发送通知")

    await event.accept()



from io import BytesIO
from pathlib import Path
from minidynamicrender.Core import DyRender
from google.protobuf.json_format import MessageToDict
from dynamicadaptor.DynamicConversion import formate_message
from bilireq.grpc.protos.bilibili.app.dynamic.v2.dynamic_pb2 import DynamicItem

from .fonts_provider import get_font


data_path = Path("data", "render")
data_path.mkdir(parents=True, exist_ok=True)


async def pil_dynamic(dyn: DynamicItem):
    dynamic: dict = MessageToDict(dyn)
    dynamic_formate = await formate_message("grpc", dynamic)
    if dynamic_formate:
        img_bio = BytesIO()
        render = DyRender(
            data_path=str(data_path),
            font_path={
                "text": str(await get_font("HarmonyOS_Sans_SC_Medium.ttf")),
                "extra_text": str(await get_font("sarasa-mono-sc-bold.ttf")),
                "emoji": str(await get_font("nte.ttf")),
            },
        )
        dynamic_render = await render.dyn_render(dynamic_formate)
        dynamic_render.convert("RGB").save(img_bio, "jpeg", optimize=True)
        return img_bio.getvalue()



import asyncio

from io import BytesIO
from wordcloud import WordCloud
from jieba.analyse.tfidf import TFIDF


from .fonts_provider import get_font

tfidf = TFIDF()


def get_frequencies(msg_list) -> dict[str, float]:
    text = "\n".join(msg_list)
    return dict(tfidf.extract_tags(text, topK=200, withWeight=True))


async def get_worldcloud_image(frequencies):
    wc = WordCloud(
        font_path=str(await get_font()),
        background_color="white",
        width=1000,
        height=800,
        repeat=False,
        random_state=42,
    )
    await asyncio.to_thread(wc.generate_from_frequencies, frequencies)
    bio = BytesIO()
    wc.to_image().save(bio, format="JPEG")
    return bio.getvalue()



import click
from ..core.announcement import PROJECT_VERSION


@click.group("BBot Cli", help="BBot 命令行工具")
@click.version_option(
    PROJECT_VERSION,
    "-v",
    "--version",
    package_name="aunly-bbot",
    prog_name="BBot",
    message="%(prog)s 当前版本：%(version)s",
    help="显示 BBot 版本",
)
@click.help_option("-h", "--help", help="显示帮助信息")
def main():
    pass


@click.command(name="run", help="运行 BBot")
@click.option("-t", "--test", is_flag=True, help="测试模式")
@click.option("-s", "--skip-verify", is_flag=True, help="跳过 MAH 可用性检查")
@click.option("-i", "--ignore-sub", is_flag=True, help="忽略登录模式下的账户订阅列表")
@click.help_option("-h", "--help", help="显示帮助信息")
def run_bot(test: bool, skip_verify: bool, ignore_sub: bool):
    from ..core import cache

    cache["test"] = test
    cache["skip_verify"] = skip_verify
    cache["ignore_sub"] = ignore_sub

    from .run import run_bot

    run_bot()


@click.command(name="api", help="运行 BBot Playwright 截图 API 服务")
@click.help_option("-h", "--help", help="显示帮助信息")
def run_api():
    import uvicorn
    from .api import app

    uvicorn.run(app, host="0.0.0.0", port=9000)


@click.command(name="install-deps", help="安装 Playwright 所需依赖")
@click.help_option("-h", "--help", help="显示帮助信息")
def install_deps():
    import sys
    from playwright.__main__ import main as pw_main

    sys.argv.append("chromium")
    pw_main()


@click.command(help="BBot 配置向导")
def config():
    from .config import click_config

    click_config()


main.add_command(run_bot)
main.add_command(run_api)
main.add_command(config)
main.add_command(install_deps)



import sys
import psutil
import richuru
import contextlib

from pathlib import Path
from loguru import logger

from .bot_config import BotConfig

from ..utils.detect_package import is_package

# read log_level and verify
log_level = str(BotConfig.log_level).upper()
if log_level not in ["TRACE", "DEBUG", "INFO", "SUCCESS", "WARNING", "ERROR", "CRITICAL"]:
    logger.error("log_level 未配置或配置无效，已自动调整为 INFO")
    log_level = "INFO"

# ensure logs dir exist
LOGPATH = Path("data", "logs")
LOGPATH.mkdir(exist_ok=True)


def in_screen():
    with contextlib.suppress(psutil.NoSuchProcess):
        for proc in psutil.Process().parents():
            if proc.name() in ["screen", "tmux", "node", "python"]:
                return proc.name() != "node" or "vscode" not in str(proc.cmdline())
    return psutil.Process().pid == 1


if in_screen() or is_package or not BotConfig.use_richuru:
    logger.info("检测到当前运行在各类容器中、运行为打包（nuitka、pyinstaller）版本或未在配置文件中启用，已禁用 richuru")
    logger.remove(0)
    logger.add(
        sys.stderr,
        level=log_level,
        backtrace=True,
        diagnose=True,
        filter=lambda record: (
            "sentry_patched_callhandlers" not in record["function"]
            or "HTTP Request" not in record["message"]
        ),
    )
else:
    richuru.install(level=log_level)

# add latest logger
logger.add(
    LOGPATH.joinpath("latest.log"),
    encoding="utf-8",
    backtrace=True,
    diagnose=True,
    rotation="00:00",
    retention="1 week",
    compression="tar.xz",
    level="INFO",
)

# add debug logger
logger.add(
    LOGPATH.joinpath("debug.log"),
    encoding="utf-8",
    backtrace=True,
    diagnose=True,
    rotation="00:00",
    retention="3 days",
    compression="tar.xz",
    level="DEBUG",
)

# add warning logger
logger.add(
    LOGPATH.joinpath("warning.log"),
    encoding="utf-8",
    backtrace=True,
    diagnose=True,
    rotation="00:00",
    retention="3 days",
    compression="tar.xz",
    level="WARNING",
)

logger.success(f"成功重载 logger，当前日志等级为 {log_level}")
logger.info(f"日志文件将会保存在 {LOGPATH} 中")

# logger.trace("TRACE 等级将会输出至控制台")
# logger.debug("DEBUG 等级将会输出至控制台")
# logger.info("INFO 等级将会输出至控制台")
# logger.success("SUCCESS 等级将会输出至控制台")
# logger.warning('WARNING 等级将会输出至控制台')
# logger.error("ERROR 等级将会输出至控制台")
# logger.critical('CRITICAL 等级将会输出至控制台')
